#ifdef USERC	; Are we using XBEE control
;Project Lynxmotion Phoenix
; Quick and dirt RC version using modified Hitec receiver (KurtE)
;====================================================================
CNT_PPM_SIGNALS	con	6
awPPMs		var	word(CNT_PPM_SIGNALS)	; Saved PPM values

DEBUG_IRQ2 con 1
#ifdef DEBUG_IRQ2
cIRQ2Trace	var	byte					; count of how many is raw..
awPPMDbg	var	word(100)				;
abIRQ2T		var	byte(100)
#endif

fPPMsValid		var	byte					; Is the PPMS stored valid?

awPPMsPrev		var	word(CNT_PPM_SIGNALS)	; Saved PPM values

;i				var byte
;fChanged		var bit

;--------------------------------------------------------------------
;[CONSTANTS]
WalkMode		con 0
TranslateMode	con 1
RotateMode		con 2
SingleLegMode	con 3
GPPlayerMode	con 4

;--------------------------------------------------------------------

;--------------------------------------------------------------------
;[InitController] Initialize the PS2 controller
;    Warning: this version has been updated to allow both XBee and
; 			PS2 to control the robot... Which ever one issues a start
;			will take over until it does a stop...
;--------------------------------------------------------------------
#ifdef USEXBEE
InitRCController:
#else
InitController:
#endif
;PS2 controller
#ifdef DOIRQ2_IN_ASM		;maybe better to debug in Basic first...
	ONASMINTERRUPT IRQ2INT, HANDLE_IRQ2
#else
	ONINTERRUPT IRQ2INT, HANDLE_IRQ2
#endif
	PMR1.bit6 = 1 ; enable pin to IRQ2 interrupt instead of normal I/O double check for arc32
	IEGR1.bit2 = 1 ; Interrupt IRQ2 on rising edge
	ENABLE IRQ2INT
	return

;==============================================================================
; [ControlRCInput] - if both controllers
; [ControlInput] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
#ifdef USEXBEE
ControlRCInput:
#else
ControlInput:
#endif
	; first pass See if we are receiving valid data...
	if cIRQ2Trace = 100 then
	
		for i = 0 to 99
			hserout [dec awPPMDbg(i),":", hex abIRQ2T(i), " "]
			if (i // 10) = 9 then
				hserout [13]
			endif
		next
		cIRQ2Trace = 101	; set so won't do anything else
	endif
	


return

;------------------------------------------------------------------------------
; IRQ2 handler - Real work done here.  The idea is to capture our current time
; 		value when the interrupt happens and then calculate a delta from the 
;		previous one.  If > some max we know that we are starting up a new run
;		of timings, else save away in the appropriate index value in our array
;		of timeing with 1 per logical channel.
;------------------------------------------------------------------------------
_lLastPulseTime	var	long			; time of the last pulse...	
_lNewPulseTime	var	long				; new pulse time
_wPulseWidth	var	word				; what is the pulse width
_bNextChannel	var	byte				; What is the next channel we are expecting

#ifdef DEBUG_IRQ2
_BIRQ2_TRACE	var byte
#endif

Handle_IRQ2:
	toggle p44				; status LED on arc32...
	
	; Use hservo
	;gosub GetCurrentTime - don't want to enable interupts yet so do inline
asm {
;	_lNewPulseTime = HServoTime 0;	
    ;calc HSERVOTIME
	mov.w	@_HSERVOTIMER:16,e1         ;6 get HSERVOTIME
	mov.w   @TCNT_0:16,r1               ;6 get TimerZ time
	bld.b	#4,@TSR_0:16					; Get the overflow status from TSR
	bcc		_GHST_NOOVERFLOW:8			; Not overflow
	inc.w	#1, e1						; do the add ourself
	mov.w   @TCNT_0:16,r1               ; nead to reload the timer value to make sure
_GHST_NOOVERFLOW:	
;	mov.l	er1, @_LNEWPULSETIME		; save away the time
	mov.l	er1, er2					; save away the new item...
	; Now lets calucalte the pulse width
	mov.l	@_LLASTPULSETIME:16, er0	; get the last time 
	sub.l	er0, er1					; calculate the difference.
	mov.l	er2, @_LLASTPULSETIME:16	; save away the new timer for later...

	mov.w	#20, r0						; we will divide by 20 to convert to uS
	divxu	r0, er1						; so now we have in uS
	mov.w	r1, @_WPULSEWIDTH:16		; Save away for now
	mov.b	#0, r4l						; DEBUG- Where does our code gooo

	cmp.w	#2500, r1					; ok see if we are in a valid range
	ble		_IRQ2_NORMAL_CHANNEL:8		; probably a normal pulse
	; Ok See where we were.  Are we expecting a long pulse
	add.b	#0x10, r4l							; debug
	mov.b	#0, r0h						; zero register
	mov.b	@_BNEXTCHANNEL:16, r0l			; Get the channel we are expecting
	cmp.b	#CNT_PPM_SIGNALS:8, r0l		; 
	bge		_IRQ2_START_NEW_PASS:8		; Normal start new pass
	add.b	#0x10, r4l							; debug
	mov.b	r0h, @FPPMSVALID:16			; error case let user know that we may not have valid data
_IRQ2_START_NEW_PASS:
	mov.b	r0h, @_BNEXTCHANNEL:16		; set the next channel to process to zero
	jmp		_IRQ2_CLEANUP:8				; go to the end to clean up
		
_IRQ2_NORMAL_CHANNEL:
	mov.b	#80,r4l						; DEBUG So we know we got here
	mov.b	@_BNEXTCHANNEL:16, r0l
	cmp.b	#CNT_PPM_SIGNALS, r0l
	bge		_IRQ2_CLEANUP:8				; Cnt exceeds our array don't save away...
	add.b	#0x10, r4l							; debug
	mov.b	r0l, r2l					; Ok lets save away the value
	extu.w	r2							; extend out index to a word	
	extu.l	er2							; could also have zeroed first instad
	mov.w	r1, @(AWPPMS,er2)			; save away in the array
	inc.b	r0l							; update our count
	mov.b	r0l,@_BNEXTCHANNEL:16		; save it away
	cmp.b	#CNT_PPM_SIGNALS, r0l
	bne		_IRQ2_CLEANUP:8				; still more to go in this pass
	add.b	#0x10, r4l							; debug
	mov.b	#1, r0l
	mov.b	r0l, @FPPMSVALID:16			; save it away	

_IRQ2_CLEANUP:
#ifdef DEBUG_IRQ2
	mov.b	@_BNEXTCHANNEL:16, r0l		; DEBUG - remember what our next channel will be...
	add.b	r0l, r4l					
	mov.b	r4l,@_BIRQ2_TRACE:16		; DEBUG - So we can figure out what the code did...
#endif	
}	

#ifdef DEBUG_IRQ2
	if cIRQ2Trace < 100 then
		awPPMDbg(cIRQ2Trace) = _WPULSEWIDTH
		abIRQ2T(cIRQ2Trace) = _BIRQ2_TRACE;
		cIRQ2Trace = cIRQ2Trace + 1
	endif
#endif


	; See if the value appears to be OK.
	if _wPulseWidth > 2500 then 
		; Where were we??? 
		if _bNextChannel < CNT_PPM_SIGNALS then
			; We got a long value when we did not expect it...???
			fPPMsValid = 0;
		endif
		_bNextChannel = 0; 	// Ok lets assume the next value will be for our first channel
	else
		if _bNextChannel < CNT_PPM_SIGNALS then
			awPPMs(_bNextChannel) = _wPulseWidth
			_bNextChannel = _bNextChannel + 1
			if  _bNextChannel = CNT_PPM_SIGNALS then
				fPPMsValid = 1		; Let the main line know that we received a complete valid set of values
			endif
		endif
	endif
		
	resume


#endif	; USERC