
; lets try to configure XBEE to run on HSERIAL
XBEEONHSERIAL con 1

;Project Lynxmotion Phoenix
;Description: Phoenix, control file.
;		The control input subroutine for the phoenix software is placed in this file.
;		Can be used with V2.0 and above
;Configuration version: V1.0
;Date: 08-01-2009
;Programmer: Jeroen Janssen (aka Xan)
;Special Thanks to: Kurt (aka Kurte) Assembly
;
;Hardware setup: DIY XBee
;
;NEW IN V1.0
;	- First Release
;
;
;DIY CONTROLS:
;	- Left Stick	(WalkMode) Body Height / Rotate
;					(Translate Mode) Body Height / Rotate body Y	
;					(Rotate Mode) Body Height / Rotate body Y
;					(Single leg Mode) Move Tars Y	
;
;	- Right Stick	(WalkMode) Walk/Strafe
;			 		(Translate Mode) Translate body X/Z
;					(Rotate Mode) Rotate body X/Z
;					(Single leg Mode) Move Tars X/Z
;
; 	- Left Slider	Speed
;	- Right Slider	Leg Lift Height
;
;	- A				Walk Mode
;	- B				Translate Mode
;	- C				Rotate Mode
;	- D				Single Leg Mode
;	- E				Balance Mode on/off
;
;	- 0				Turn on/off the bot
;
;	- 1-8			(Walk mode) Switch gaits
;	- 1-6			(Single leg mode) Switch legs
;
;====================================================================
;[DEBUG]
;DEBUG_OUT con 1
'DEBUG_VERBOSE con 1
;DEBUG_ENTERLEAVE con 1
;--------------------------------------------------------------------
;[CONSTANTS]
WalkMode		con 0
TranslateMode	con 1
RotateMode		con 2
SingleLegMode	con 3
GPPlayerMode	con 4

;--------------------------------------------------------------------

;[DIY XBee Controller Variables]
LastButton 			var Byte(2)		; The state of the buttons in previous packet
iNumButton			var	byte		; which button is pressed. Only used in a few places.
fGPSequenceRun		var	bit			; Was a sequence started?

fPacketChanged		var bit			; Something change in the packet
bPacketPrev			var	byte(XBEEPACKETSIZE)

i					var	byte
b					var	byte		; 
pT					var	pointer		; temporary pointer


; Feedback messages - First byte is the count
_WALKMODE			bytetable	7,  "Walking"
_TANSLATEMODE		bytetable	14, "Body Translate"
_ROTATEMODE			bytetable	11, "Body Rotate"
_SINGLELEG			bytetable	10, "Single Leg"
_BALANCEON			bytetable	10, "Balance On"
_BALANCEOFF			bytetable	11, "Balance Off"
_GPSEQMODE			bytetable	12, "Run Sequence"
_GPSEQSTART			bytetable	14, "Start Sequence"
_GPSEQCOMPLETE		bytetable	 9, "Completed"
_GPSEQDISABLED		bytetable	12, "Seq Disabled"
_GPSEQNOTDEF		bytetable	15, "Seq Not defined"

; Feedback - Gait names *** Warning: make sure counts match as I hop through the strings"
_GATENAMES			bytetable	8, 	"Ripple 6",|
								9, 	"Ripple 12",|
								12,	"Quadripple 9",|
								8, 	"Tripod 4",|
								8, 	"Tripod 6",|
								8, 	"Tripod 8",|
								7, 	"Wave 12",|
								7, 	"Wave 18"
; Other strings...
_EER_SSC1			bytetable	"EER -" ; 5 characters
_EER_SSC2			bytetable	";2",13 ; 3

; Define a sound to send back...
_GPSNDERR			bytetable 100, 5000/25,80, 4000/25] ' play it a little different...

;
; Each of the packets will be defined throughout this file...
;
ControlMode			var byte
abSeqPtr			var byte(2)
;--------------------------------------------------------------------
InitController:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Init Controller", 13]
#endif	


high p4
high p5
high p6

	gosub InitXBee
   
 	ControlMode = WalkMode
	fGPSequenceRun = 0		; make sure it is init...
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Init Controller", 13]
#endif	
 
return


;==============================================================================
; [Control] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
	
ControlInput:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Control Input", 13]
#endif	
	; Not sure the best place to test this, but if we ran a sequence and now
	; it says it is not running, let the user know this.
	if fGPSequenceRun and (GPStart = 0) then
		fGPSequenceRun = 0		; clear out the data...
		gosub XBeeOutputString[@_GPSEQCOMPLETE]
		GOSUB XBeeResetPacketTimeout ; sets _lTimerLastPacket
		
	endif

	; First lets try getting a packet from the XBEE
	gosub ReceiveXBeePacket
	; See if we have a valid packet to process
	if fPacketValid  then
		fPacketChanged = 0	; don't need to worry about slop 
		for i = 0 to XBEEPACKETSIZE-1
			if (bPacket(i) <> bPacketPrev(i)) then
				fPacketChanged = 1
				bPacketPrev(i) = bPacket(i)
			endif
		next
		
		if fPacketChanged  then
			if fPacketForced then
				gosub SendXbeeNewDataOnlyPacket[1]
			endif
#ifdef DEBUG_OUT
			serout s_out, i9600, [bin8 bPacket(PKT_BTNHI)\8, bin8 bPacket(PKT_BTNLOW)\8, " "]
			for i = 2 to 7
				serout s_out, i9600, [dec bPacket(i)," "]
			next
			serout s_out, i9600,[13] 
#endif

		endif

		; OK lets try "0" button for Start. 
		IF bPacket(PKT_BTNLOW).bit0 and (LastButton(0).bit0 = 0) THEN	;Start Button (0 on keypad) test
			IF(HexOn) THEN
				'Turn off
				Sound P9,[100\5000,80\4500,60\4000]
				BodyPosX = 0
				BodyPosY = 0
				BodyPosZ = 0
				BodyRotX1 = 0
				BodyRotY1 = 0
				BodyRotZ1 = 0
				TravelLengthX = 0
				TravelLengthZ = 0
				TravelRotationY = 0
				
				SSCTime = 600
				GOSUB ServoDriverStart
				GOSUB ServoDriverCommit
				HexOn = 0
#ifdef DEBUG_MAIN				
				serout s_out, i9600, ["Cycles: ", dec cCycleTime, " Sum:", dec lsumCycleTime, 13]
#endif				
			ELSE
				'Turn on
				Sound P9,[60\4000,80\4500,100\5000]
				SSCTime = 200
				HexOn = 1	
#ifdef DEBUG_MAIN				
			  	lsumCycleTime = 0 ;;; bugbug
	  			cCycleTime = 0
#endif
			ENDIF
		ENDIF	
		
		IF HexOn THEN
			IF bPacket(PKT_BTNHI).bit2 and (LastButton(1).bit2 = 0) THEN		;A Button Walk Mode
				sound p9, [50\4000]
				ControlMode = WalkMode
				gosub XBeeOutputString[@_WALKMODE]
			ENDIF

			IF bPacket(PKT_BTNHI).bit3 and (LastButton(1).bit3 = 0) THEN		;B Button Translate Mode
				sound p9, [50\4000]
				ControlMode = TranslateMode
				gosub XBeeOutputString[@_TANSLATEMODE]
			ENDIF
      
			IF bPacket(PKT_BTNHI).bit4 and (LastButton(1).bit4 = 0) THEN		;C Button Rotate Mode
				sound p9, [50\4000]
				ControlMode = RotateMode
				gosub XBeeOutputString[@_ROTATEMODE]
			ENDIF
      
			IF bPacket(PKT_BTNHI).bit5 and (LastButton(1).bit5 = 0) THEN		;D Button Single Leg Mode
				sound p9, [50\4000]
				IF SelectedLeg=255 THEN ; none
					SelectedLeg=cRF
				ELSEIF ControlMode=SingleLegMode ;Double press to turn all legs down
					SelectedLeg=255 ; none
				ENDIF
				ControlMode=SingleLegMode        
				gosub XBeeOutputString[@_SINGLELEG]
			ENDIF

			IF bPacket(PKT_BTNHI).bit6 and (LastButton(1).bit6 = 0) THEN		; E Button Balance Mode on/of
				IF BalanceMode = 0 THEN
					BalanceMode = 1
					sound P9,[100\4000, 50\8000]
					gosub XBeeOutputString[@_BALANCEON]
				ELSE
					BalanceMode = 0
					sound P9,[250\3000]
					gosub XBeeOutputString[@_BALANCEOFF]
				ENDIF  
			ENDIF   

			IF bPacket(PKT_BTNHI).bit7 and (LastButton(1).bit7 = 0) THEN		; F Button GP Player Mode Mode on/off
      			if GPEnable THEN ;F Button GP Player Mode Mode on/off -- SSC supports this mode
					gosub XBeeOutputString[@_GPSEQMODE]
					sound P9, [50\4000]
					
					BodyPosX = 0
					BodyPosZ = 0
					BodyRotX1 = 0
					BodyRotY1 = 0
					BodyRotZ1 = 0
					TravelLengthX = 0
					TravelLengthZ = 0
					TravelRotationY = 0
					
					SelectedLeg=255 ; none
					SLHold=0
			
					ControlMode = GPPlayerMode
				else
					gosub XBeeOutputString[@_GPSEQDISABLED]
					sound P9, [50\4000]
				endif
			ENDIF

			; Hack there are several places that use the 1-N buttons to select a number as an index
			; so lets convert our bitmap of which key may be pressed to a number...
			; BUGBUG:: There is probably a cleaner way to convert...
			iNumButton = 0		; assume no button
			if ((bPacket(PKT_BTNLOW) & 0xFE) or (bPacket(PKT_BTNHI) & 0x03)) and ((LastButton(0) & 0xFE) = 0) and ((LastButton(1) & 0x03) = 0) then
				b = bPacket(PKT_BTNLOW) & 0xfe
				if b then
					while not b.bit0
						b = b >> 1
						iNumButton = iNumButton + 1
					wend
				else
					b = bPacket(PKT_BTNHI)
					iNumButton = 8		; start off at bit 8
					while not b.bit0
						b = b >> 1
						iNumButton = iNumButton + 1
					wend
				endif
			endif
			; BUGBUG:: we are using all keys now, may want to reserve some...		
			;Switch gait
			; We will do slightly different here than the RC version as we have a bit per button
			IF ControlMode=WalkMode and iNumButton  and (iNumButton <= 8) THEN	;1-8 Button Gait select	  
				IF ABS(TravelLengthX)<cTravelDeadZone & ABS(TravelLengthZ)<cTravelDeadZone & ABS(TravelRotationY*2)<cTravelDeadZone THEN
				
					;Switch Gait type
					GaitType = iNumButton-1
					Sound P9,[50\4000]
					GOSUB GaitSelect
					; For the heck do in assembly!
					pT = @_GATENAMES	; Point to the first of the paint names
					mov.l	@PT:16, er0					; pointer to our strings
					mov.b	@GAITTYPE, r1l				; r1l has the gait type
					beq		_CI_GS_ENDLOOP:8			; passed zero in so done
_CI_GS_LOOP:					
					mov.b	@er0+, r2l					; get the count of bytes in this string and increment to next spot
					extu.w	r2
					extu.l	er2							; convert to 32 bits
					add.l	er2, er0					; and increment to the next string
					dec.b	r1l							; decrement our count
					bne		_CI_GS_LOOP:8				; done
_CI_GS_ENDLOOP:
					mov.l	er0, @PT					; save away our generated pointer					
#if 0					
					pT = @_GATENAMES	; Point to the first of the paint names
					
					i = GaitType
					while i > 0
						b = @pT		; get the count of bytes for the string
						pT = pT + b + 1	; increment to the start of the next string
						i = i - 1
					wend
#endif					
					; And tell the remote the name for the selected gate
#ifdef DEBUG_OUT
					serout s_out, i9600, ["GT - Addrs: ", hex @_GATENAMES, 13]
					serout s_out, i9600, ["GT: ", dec GaitType, " ", hex pT, "=", hex @pt, 13]
#endif					
					
					gosub XBeeOutputString[pT]
					
				ENDIF
			ENDIF

			;Switch single leg
			IF ControlMode=SingleLegMode THEN	  
				IF iNumButton>=1 & iNumButton<=6 THEN
					Sound P9,[50\4000]
					SelectedLeg = iNumButton-1
					SLHold=0
				ENDIF
			
				IF iNumButton = 9  THEN ;Switch Directcontrol
			  		sound p9, [50\4000]
			  		SLHold = SLHold^1 	;Toggle SLHold
				ENDIF
			ELSEIF ControlMode=Walkmode
				SelectedLeg=255	; none
				SLHold=0
			ENDIF

			;Body Height
			BodyPosY = bPacket(PKT_LJOYUD) / 2
			
			;Leg lift height - Right slider has value 0-255 translate to 30-93
			LegLiftHeight = 30 + bPacket(PKT_RSLIDER)/5
			
			;Walk mode			
			IF (ControlMode=WalkMode) THEN 			
				TravelLengthX = -(bPacket(PKT_RJOYLR) - 128)
				TravelLengthZ = -(bPacket(PKT_RJOYUD) - 128)	
				TravelRotationY = -(bPacket(PKT_LJOYLR) - 128)/3

			ENDIF
			
			;Body move	
			IF (ControlMode=TranslateMode) THEN	
				BodyPosX = (bPacket(PKT_RJOYLR)-128)/2
				BodyPosZ = (bPacket(PKT_RJOYUD)-128)/2
				BodyRotY1 = (bPacket(PKT_LJOYLR)-128)*2
			ENDIF		
			
			;Body rotate	
			IF (ControlMode=RotateMode) THEN	
				BodyRotX1 = (bPacket(PKT_RJOYUD)-128)*2
				BodyRotY1 = (bPacket(PKT_LJOYLR)-128)*2
				BodyRotZ1 = -(bPacket(PKT_RJOYLR)-128)*2
			ENDIF
			
			;Single Leg Mode
			IF (ControlMode = SingleLegMode) THEN
				SLLegX = (bPacket(PKT_RJOYLR)-128)
				SLLegZ = -(bPacket(PKT_RJOYUD)-128)
				SLLegY = -(bPacket(PKT_LJOYLR)-128)/5
			ENDIF
			
			IF (ControlMode = GPPlayerMode & iNumButton>=1 & iNumButton<=9 ) THEN	;1-9 Button Play GP Seq
				IF (GPStart = 0) THEN
					GPSEQ = iNumButton-1
				ENDIF
				; Lets verify that there is a sequence number defined for the specified sequence. We can ask the SSC-32
				; for the pointer to the sequence, if 0 or -1 we know it is not defined.
				abSeqPtr = 0,0		; assume no sequence
#ifndef SSC_USE_TASERIAL
				serout cSSC_OUT, cSSC_BAUD, ["EER -", dec GPSEQ*2, ";2",13]
				serin cSSC_IN, cSSC_BAUD, 1000, _TO_GPSP, [str abSeqPtr\2] ;read in 2 byte header
_TO_GPSP:				
#else
				gosub TASerOut[cSSC_OUT, cSSC_BAUD, @_EER_SSC1, 5]		; "EER -"
				gosub SSCOutputNum[GPSEQ*2]								; output the number	
				gosub TASerOut[cSSC_OUT, cSSC_BAUD, @_EER_SSC2, 3]		; ";2",13
				gosub TASerIn[cSSC_IN, 0, cSSC_BAUD, @abSeqPtr, 2, 10000, -1], cGPBytes
#endif				
				IF ((abSeqPtr(0) = 0) and (abSeqPtr(1) = 0)) or ((abSeqPtr(0) = 0xff) and (abSeqPtr(1) = 0xff))	THEN
					gosub XBeeOutputString[@_GPSEQNOTDEF]  ; that sequence was not defined...
					gosub XBeePlaySounds[@_GPSNDERR, 4];	// BUGBUG: maybe should pass number of notes instead?
				ELSE
					; let user know that sequence was started
					gosub XBeeOutputString[@_GPSEQSTART]  ; Tell user sequence started.
#ifdef XBEEONHSERIAL
					gosub WaitHseroutComplete			; the communications interrupts may cause problems.
#endif					
					fGPSequenceRun = 1 ; remember that ran one...
					GPStart = 1
				ENDIF
			ENDIF
			
			;Calculate walking time delay
			InputTimeDelay = 128 - (ABS((bPacket(PKT_RJOYLR)-128)) MIN ABS(((bPacket(PKT_RJOYUD)-128)))) MIN ABS(((bPacket(PKT_LJOYLR)-128))) + (128 -(bPacket(PKT_LSLIDER))/2)


		ENDIF
	  
		LastButton(0) = bPacket(PKT_BTNLOW)
		LastButton(1) = bPacket(PKT_BTNHI)
	else
		; Not a valid packet - we should go to a turned off state as to not walk into things!
		IF(HexOn and (fPacketForced or fPacketEnterSSCMode)) THEN
			'Turn off
			Sound P9,[100\5000,80\4500,100\5000,60\4000] ' play it a little different...
			BodyPosX = 0
			BodyPosY = 0
			BodyPosZ = 0
			BodyRotX1 = 0
			BodyRotY1 = 0
			BodyRotZ1 = 0
			TravelLengthX = 0
			TravelLengthZ = 0
			TravelRotationY = 0
			
			SSCTime = 600
			GOSUB ServoDriverStart
			GOSUB ServoDriverCommit
			HexOn = 0
		endif
		if fPacketEnterSSCMode then
			' received a packet telling us to enter into SSC-mode.
			' if the hex was on we basically released everything, so now
			' simply cal off to the SSC mode handler
			gosub XBeeHandleSSCMode		' 
		endif
	endif

#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Control Input", 13]
#endif	

return	

