;==============================================================================
; MechBrat Xbee version
; Program written for Bot Board II, Basic Atom Pro 28, Basic Micro Studio
;Written by Nathan Scherdin, Jim and James Frye
; Mucked up (alot)by Kurt
;
;KNOWN BUGS: 
;   - Probably Alot ;) 
; ###### WARNING ##### Work in progress use at your (and your robots) own risk!
;==============================================================================
XBEEONHSERIAL con 1

DEBUG_OUT con 1
;DEBUG_VERBOSE con 1
; [Defines]
TRUE con -1
FALSE con 0

BUTTON_DOWN con 0
BUTTON_UP 	con 1

#ifdef TMA
BAP28	con	1
#else
BAP40	con	1		; define something for BAP40
#endif


;GUNS_CONNECTED con 1

;System variables 
;==============================================================================
; [IO Definitions] Lets define what each IO line is used for.
;==============================================================================
;LGswP		con p19
LGUNP		con p18
;RGswP		con p17
RGUNP		con p16

;TXD		con p15	; used by hserial to xbee
;RXD		con p14 ; used by hserial to xbee
Turret		con p11
righthip	con p10
;Speaker 		p9 
rightknee	con p8 

rightankle	con p12	; moved from P7 - to allow Flow control on lower voltage pin

lefthip		con p6 
leftknee	con p5 
leftankle	con p4 

IRScanPin	con p1
LaserPin 	con p0

;==============================================================================
; [EEPROM] Define EEPROM memory locations
;==============================================================================
; Our Mechbrat servo offsets program, tries to save away the servo offsets into
; EEPROM, which we will retrieve.  If this fails we will 
NUMSERVOS		con	7
aServoOffsets		var	sword(NUMSERVOS)				
righthip_start 		var	aServoOffsets(0)
rightknee_start 	var	aServoOffsets(1)
rightankle_start 	var	aServoOffsets(2)
lefthip_start 		var	aServoOffsets(3)
leftknee_start 		var	aServoOffsets(4)
leftankle_start 	var	aServoOffsets(5)
turret_start		var	aServoOffsets(6)


; Array of servo numbers to check for busy in the movement function
;----------------- Simple test receiving data from Xbee -----------------------
lCurrentTimeT		var	long		; 
lTimerLastPacket	var	long		; the timer value when we received the last packet
lTimerHSerinStart	var	long		; BUGBUG:: trying to work around timeouts?

bHSerinHasData		var	byte		; 

fPacketValidPrev	var	bit
i					var	byte
b					var	byte		; 

;----------------- Other variables --------------------------------------------
fPacketChanged		var bit		; we had a channel change
bPacketPrev			var	byte(8)

;==============================================================================
; [XBee definitions]
; We are also setting up the remote to have several different display and/or 
;==============================================================================


;==============================================================================
; [Command Quick Reference]
;==============================================================================
;- Command 0 = Home Position           -;
;- Command 1 = Walk Forward            -;
;- Command 2 = Walk Backward           -;
;- Command 4 = Kick (Left)             -;
;- Command 5 = Kick (Right)            -;
;- Command 6 = Headbutt                -;
;- Command 7 = Get up from Front       -;
;- Command 8 = Get up from Back        -;
;- Command 9 = Rest Position           -;
;- Command 11= Turn Left               -;
;- Command 12= Turn Right              -;
;- Command 13= Slight Turn Left        -;
;- Command 14= Slight Turn Right       -;

command 				var byte 	; Command choosen by the input function
fAbortCommand 			var	byte	; Should the current command be aborted?

;==============================================================================
; [General defines and variables]
;==============================================================================
TravelDeadZone			con 4		; The deadzone for the analog input from the remote

stepsperdegree1 		con 1666 	; calibrate steps per degree. 

IdleBot 				var word
;WalkSpeed1				var slong
WalkAngle1				var slong
LHipAngle				var sword
RHipAngle				var sword

TurretAngle				var slong
TravLength				var byte
LastStep				var byte
AnkleAdj				var byte

LaserOn 				var bit		; Is the Laser turned on?
BotActive 				var bit		; Is the robot currently on (Logical Start button)
LGunWaitingToCycle		var	bit		; Are we waiting for the gun to fire to turn it off
RGunWaitingToCycle 		var	bit		;

idle 					var byte	; Used in GetHServo calls to see if servo is idle
notidle					var	byte	; Used to see if all of the servos are idle or not
junk 					var word	; used as a don't care in calls to GetHServo

LastButton 				var Byte(2)	; Save the state of the two bytes of buttons from last call
JoyStickValueIn			var slong	; Used to process joystick input data...

;[Code timing]
wCurTimer				var	word	; Last used 
#ifdef DEBUG_VERBOSE
wMovementCalls			var	word	; number of times we called movement
WMovCallsHServo 		var word	; Number of times called that HSERVO was called from callback
lMovementDTSum			var	long	; time taken up in movement function in Us... 
lMovDTSumHServo			var	long	; time taken up in movement function in Us... 
CBHervoCalled			var	bit		; was hservo called in call back
#endif


;--------------------------------------------------------------------
;[Init]

; Initialize some of the system state variables
TurretAngle = 0
TravLength = 6
LastStep = 0
AnkleAdj = 0
LaserOn = FALSE
BotActive = FALSE
fSendOnlyNewMode = FALSE
LGunWaitingToCycle = FALSE
RGunWaitingToCycle = FALSE
LastButton(0) = 0
LastButton(1) = 0

; Enable HServo and Interrupts
ENABLEHSERVO 
ENABLE

gosub InitController
gosub SendXbeeNewDataOnlyPacket[1]		; tell system to go in new mode.


; make sure the servo offsets are set
aServoOffsets = rep 0\NUMSERVOS		; Use the rep so if size changes we should properly init
gosub ReadServoOffsets


;Init positions 
;Note, movement subroutine arguments are Rightankle,Rightknee,Righthip,Leftankle,Leftknee,Lefthip,speed 
gosub movement [  0,  0,  0,  0,  0,  0,  0] 
hservo [Turret\TurretAngle + turret_start\0] ; get the turret into center position.

;Gun/Laser Initialization
hservo [RGUNP\0\0,LGUNP\0\0,LaserPin\0\0]
;input RGSWP
;input LGSWP

pause 1000 

 
sound 9,[50\4000,40\3500,40\3200,50\3900]

gosub InitTimer

#ifdef DEBUG_VERBOSE
; Timer variables used if we are debugging the timings.
wMovementCalls = 0
lMovementDTSum = 0
WMovCallsHServo = 0
lMovDTSumHServo = 0
#endif

      

;==============================================================================
;[Main Loop]
;==============================================================================

main:
	gosub ControlInput[TRUE]
	gosub CheckGuns

	; The Input functions will return a Command number, that we will then call move on.
	; We do this here instead of in the Input function as the input function will be called from within the 
	; move function, and this will remove some recursion and hopefully keep more consistent states
	; first check to see if we need to do idle processing...
	if command <> 0xff	then
		if(IdleBot = 1000)then
			command = 9
		elseif(IdleBot = 50)
			AnkleAdj = 0
		else
			IdleBot = IdleBot + 1
		endif
	endif

	; 
	if command <> 0xff	then
		gosub move
'	else
'		pause 50
	endif

goto main 

;==============================================================================
;[Move] - The move function is called from the main loop.  It handles the different commands that
; 		are mostly generated by the PS2 input.  The main value that comes into this function is the
;		variable command, which tells us what to do.  
;
;	Warning: the movement function has been converted from floating point to integer values where the
;            the different servo positions are in 1/10 of a degree.  IE 90 degrees would be 900
;==============================================================================

move:
	fAbortCommand = FALSE 							; Reset the abort command
	if(command = 1) then							; Walk
		if LastStep = 0 then
			gosub movement [  WalkAngle1, WalkSpeed1, WalkSpeed1, -WalkAngle1,-WalkSpeed1,-WalkSpeed1,500] 
			gosub movement [ -WalkAngle1, WalkSpeed1, WalkSpeed1,  WalkAngle1,-WalkSpeed1,-WalkSpeed1,500]
			LastStep = 1
		elseif LastStep = 1
			gosub movement [ -WalkAngle1,-WalkSpeed1,-WalkSpeed1,  WalkAngle1, WalkSpeed1, WalkSpeed1,500] 
			gosub movement [  WalkAngle1,-WalkSpeed1,-WalkSpeed1, -WalkAngle1, WalkSpeed1, WalkSpeed1,500] 
			LastStep = 0
		endif
		AnkleAdj = AnkleAdj + 10
		if (AnkleAdj > 60) then
			AnkleAdj = 60
		endif
      elseif(command = 4)							; Kick Left
         gosub movement [  0,   0,  0,  0,   0,   0, 500] 
         gosub movement [-140,  0,  0, 420,  0,   0, 500] 
         gosub movement [-230,  0,  0,  0,-320, 410, 500] 
         gosub movement [-230,  0,  0,  0, 240,-200, 250] 
         gosub movement [-180,  0,  0,  0,   0,   0, 500] 
         gosub movement [   0,  0,  0,  0,   0,   0, 500]
      elseif(command = 5)							; Kick Right
         gosub movement [   0,   0,   0,  0,   0,  0, 500] 
         gosub movement [ 420,   0,   0,-140,  0,  0, 500] 
         gosub movement [   0,-320, 410,-230,  0,  0, 500] 
         gosub movement [   0, 240,-200,-230,  0,  0, 250] 
         gosub movement [   0,   0,   0,-180,  0,  0, 500] 
         gosub movement [   0,   0,   0,   0,  0,  0, 500] 
      elseif(command = 7)							; Get up from front 
         gosub movement [   0,   0,   0,  0,   0,   0, 500] 
         gosub movement [   0, 900, 450,  0, 900, 450, 500] 
         gosub movement [ 400, 900,-370,  0, 900, 450, 500] 
         gosub movement [   0, 900,-650,  0, 900,-650, 500] 
         if not fAbortCommand then
         	pause 300
         endif
		 gosub movement [  0,   0,   0,   0,   0,   0,1000] 
      elseif(command = 8)							; Get up from back 
         gosub movement [  0,   0,   0,  0,   0,  0,1000] 
         gosub movement [  0,-900,  50, 00,-900,  50,1000] 
         gosub movement [  0,   0,   0,  0,   0,  0,1000] 
;         pause 1000 
;         gosub movement [  0, 220,-800, 00, 220,-800,1000] 
;         gosub movement [  0,  0,  0,  0,  0,  0,1000]
       elseif(command = 0)							; Home Position 
         gosub movement [  0,  0,  0,  0,  0,  0, 500] 
;         sound 9,[50\4400] 
;         pause 50 
;         sound 9,[50\4400] 
;         pause 50 
;         sound 9,[50\4400] 
      elseif(command = 6)							; Headbutt 
         gosub movement [  0,   0,   0,  0,   0,   0, 500] 
         gosub movement [  0,-500,-900,  0,-500,-900, 500] 
         gosub movement [  0, 320,-580,  0, 320,-580, 400] 
         if not fAbortCommand then
         	pause 200
         endif
         gosub movement [  0,-110,  70,  0,-110,  70, 500] 
         gosub movement [  0,   0,   0,  0,   0,   0, 500] 
;         sound 9,[50\4400] 
;         pause 50 
;         sound 9,[50\3960] 
;         pause 50 
;         sound 9,[50\3960] 
      elseif(command = 11)							; Turn left
         gosub movement [ 200,   0,   0,-140,   0,   0,500] 
         gosub movement [ 200,-350,-350,-140, 350, 350,500] 
         gosub movement [   0,-350,-350,   0, 350, 350,500] 
         gosub movement [   0, 350, 350,   0,-350,-350,500] 
         gosub movement [ 200, 350, 350,-140,-350,-350,500] 
         gosub movement [ 200,   0,   0,-140,-350,-350,500]
         gosub movement [-140,   0,   0, 200,-350,-350,500]
         gosub movement [-140,   0,   0, 200,   0,   0,500]
         gosub movement [   0,   0,   0,   0,   0,   0,500]
      elseif(command = 12)							; Turn right
         gosub movement [-140,   0,   0, 200,  0,   0,500] 
         gosub movement [-140, 350, 350, 200,-350,-350,500] 
         gosub movement [   0, 350, 350,   0,-350,-350,500] 
         gosub movement [   0,-350,-350,   0, 350, 350,500] 
         gosub movement [-140,-350,-350, 200, 350, 350,500] 
         gosub movement [-140,-350,-350, 200,   0,   0,500]
         gosub movement [ 200,-350,-350,-140,   0,   0,500]
         gosub movement [ 200,   0,   0,-140,   0,   0,500]
         gosub movement [   0,   0,   0,   0,   0,   0,500]
      elseif(command = 13)							;coarse correct left
         gosub movement [   0,   0,   0,   0,   0,   0,500]
         gosub movement [ 250,   0,   0,-180,   0,   0,500]
         if not fAbortCommand then
         	pause 100
         endif
         gosub movement [  20,-200,-200,-180, 200, 200,500]
         gosub movement [   0,-200,-200,   0, 200, 200,500]
         gosub movement [   0,   0,   0,   0,   0,   0,500]

      elseif(command = 14)							;coarse correct right
         gosub movement [   0,   0,   0,   0,   0,   0,500]
         gosub movement [-180,   0,   0, 250,   0,   0,500]
         if not fAbortCommand then
         	pause 100
         endif
         gosub movement [-180, 200, 200,  20,-200,-200,500]
         gosub movement [   0, 200, 200,   0,-200,-200,500]
         gosub movement [   0,   0,   0,   0,   0,   0,500]
      elseif(command = 9)							; Rest Position 
         gosub movement [   0, 350, 400,   0, 350, 400,500] 
      endif
   IdleBot = 0
return   


;==============================================================================
;[CheckGuns] - This function will simply check the state of the two guns and if the switch
;	in a gun changes state then we will turn the gun off...  This is called both
; 	from the main input loop as well as from the Movement Callback function which is called
; 	while a movement is being done....
;==============================================================================
RSwitch	var	bit
LSwitch var	bit
CheckGuns:
#ifdef GUNS_CONNECTED
	RSwitch = in17
	LSwitch = in19

	; If the Gun is waiting to fire is set then we know we released the button 
	if RGunWaitingToCycle and (RSwitch = 0) then
		hservo [RGUNP\-7000\0]
		RGunWaitingToCycle = FALSE
	endif
		
	if LGunWaitingToCycle and (LSwitch = 0) then
		hservo [LGUNP\-7000\0]
		LGunWaitingToCycle = FALSE
	endif
#endif
return




;==============================================================================
;[MovementCB] = This callback will be called by movement in a loop in between when HServo is called and
; while it waits for the Hservo to complete.  The function has the ability to abort the wait and potentially
; other calls to movement by setting the variable fAbortCommand.  Note Movement will not clear this flag, as
; this will allow whole sequences to be aborted easily.
;==============================================================================
MovementCB:
	; add call back once I have the basics working...
	gosub ControlInput[0]
	gosub CheckGuns

return


;==============================================================================
; [InitController] - This function will attempt to initialize the XBee, it will check
; 			to see the power level of the VS to see if the power is turned on
;			if so it will init the XBee to the proper startup levels
;==============================================================================
InitController:
	gosub InitXBee
	return


;==============================================================================
; [ControlInput] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
fMainInput var byte		; will be non-zero for main loop, 0 for secondary movement calls...
	
ControlInput[fMainInput]:
	; Init to say command is 0xff - For now only when we are main input
	if FMainInput then
		command = 0xff
	endif

	; First lets try getting a packet from the XBEE
	gosub ReceiveXBeePacket
	; See if we have a valid packet to process
	if fPacketValid  then
		fPacketChanged = FALSE	; don't need to worry about slop 
		for i = 0 to 7
			if (bPacket(i) <> bPacketPrev(i)) then
				fPacketChanged = TRUE
				bPacketPrev(i) = bPacket(i)
			endif
		next
		
		if fPacketChanged  then
			if fPacketForced then
				gosub SendXbeeNewDataOnlyPacket[1]
			endif

#ifdef DEBUG_OUT
			serout s_out, i9600, [bin8 bPacket(PKT_BTNHI)\8, bin8 bPacket(PKT_BTNLOW)\8, " "]
			for i = 2 to 7
				serout s_out, i9600, [dec bPacket(i)," "]
			next
			serout s_out, i9600,[13] 
#endif
		endif

		; OK lets try "0" button for Start. 
		IF bPacket(PKT_BTNLOW).bit0 and (LastButton(0).bit0 = 1) THEN	;Start Button (0 on keypad) test
			if FMainInput then
				IF(BotActive) THEN
					'Turn off
					Sound P9,[100\4400,80\3800,60\3200]
					command = 9
					;gosub move
					BotActive = False
					; output debug stuff about timeings...
#ifdef DEBUG_VERBOSE
					serout s_out, i9600, ["Calls: ", dec wMovementCalls, " Tot: ", dec lMovementDTSum, " Avg: ", dec lMovementDTSum/wMovementCalls, 13]
					serout s_out, i9600, ["Calls HServo : ", dec WMovCallsHServo, " Tot: ", dec lMovDTSumHServo, " Avg: ", dec lMovDTSumHServo/WMovCallsHServo, 13]
					wMovementCalls = 0
					lMovementDTSum = 0
					WMovCallsHServo = 0
					lMovDTSumHServo = 0
#endif         
					
				ELSE
					'Turn on
					Sound P9,[60\3200,80\3800,100\4400]
					command = 0
					BotActive = True	
				ENDIF
			else
				; If the start button is pressed while a movemet is active we will try to abort out of the current
				; movement.  We won't update the LastButton state below, so we should then go through here again
				; from the main loop to then go to the Turn off state.
				fAbortCommand = TRUE	; this should abort us out of the movement
			endif
		ENDIF	
		
		IF BotActive THEN
		
			; Some commands we will only process if we are called from the main loop.
			IF fMainInput then		; must of the commands will only be processed by the main input
			
				; We will use the Right Slider for this one...
				; May put more resolution later, but below 128->6 above->8
				if bPacket(PKT_RSLIDER) > 128 then
					if TravLength <> 8 then
						TravLength = 8
						Sound P9,[80\3800,60\3200]
					endif
				else
					if TravLength <> 6 then
						TravLength = 6
						Sound P9,[80\3200,60\3800]
					endif
				ENDIF
				
				IF bPacket(PKT_BTNHI).bit2 and (LastButton(1).bit2 = 0) THEN	;Up(A) Button test
					command = 7
				ENDIF

				IF bPacket(PKT_BTNHI).bit3 and (LastButton(1).bit3 = 0) THEN	;Down(B) Button test
					command = 8
				ENDIF
			
				IF bPacket(PKT_BTNLOW).bit2 and (LastButton(0).bit2 = 0) THEN	; 2Button test
					command = 0
				ENDIF	
			
				
				; Note, Speed and Angle here are in 10s so multiply by 10...		
				IF (ABS(bPacket(PKT_RJOYUD)-128) > TravelDeadZone) THEN
					JoyStickValueIn = bPacket(PKT_RJOYUD) - 128
					if JoyStickValueIn > 0 then
						WalkSpeed1 = (JoyStickValueIn*10)/TravLength
						WalkAngle1 = (JoyStickValueIn + 40) - AnkleAdj
					else;if (bPacket(PKT_LJOYUD)-128) < 0
						WalkSpeed1 = (JoyStickValueIn*10)/TravLength
						WalkAngle1 = -((JoyStickValueIn - 40) + AnkleAdj)
					endif
					command = 1
				
				ELSEIF (ABS(bPacket(PKT_RJOYLR)-128) > (TravelDeadZone*4))
					JoyStickValueIn = bPacket(PKT_RJOYLR) - 128
					if (JoyStickValueIn > TravelDeadZone) AND (JoyStickValueIn < 70) then
						command = 14
					elseif (JoyStickValueIn >= 70)
						command = 12
					elseif (JoyStickValueIn < -TravelDeadZone) AND (JoyStickValueIn > -70)
						command = 13
					else
						command = 11
					endif
				ENDIF
				
					; BUGBUG: We will allow the other right UP/Down joystick change the hip angles to allow us to 
					; shoot.  Currenly only when we are not walking, but may change to instead of setting an independent
					; value we will instead build an offset from the movements.  But would probably need a button to press
					; to go back to zero...
					IF (ABS(bPacket(PKT_LJOYUD)-128) > TravelDeadZone) THEN
								
						RHipAngle = RHipAngle +((bPacket(PKT_LJOYUD) -128)*3)
						IF RHipAngle > 6000 THEN
							RHipAngle = 6000
						ELSEIF RHipAngle < -6000 
							RHipAngle = -6000
						ENDIF		
						LHipAngle = LHipAngle -((bPacket(PKT_LJOYUD) -128)*3)
						IF LHipAngle > 6000 THEN
							LHipAngle = 6000
						ELSEIF LHipAngle < -6000 
							LHipAngle = -6000
						ENDIF		
				
						hservo [righthip\(RHipAngle + righthip_start)\0,lefthip\(LHipAngle + lefthip_start)\0]
						IdleBot = 0	
		
					ENDIF
			ENDIF
			;
			; some commands can be processed in the secondary loop...
			;
			; check to see if we wish to toggle the send only new data mode...
			IF bPacket(PKT_BTNHI).bit6 and (LastButton(1).bit6 = 0) THEN		; (E) Experiment: Only send data when it changes
				; toggle the Send only new data mode
				if not fSendOnlyNewMode then
					gosub SendXbeeNewDataOnlyPacket[1]
					sound 9,[40\3200,50\3900]
				else
					gosub SendXbeeNewDataOnlyPacket[0]
					sound 9,[50\3900,40\3200]
				endif			
			ENDIF 

			; Check to see if we are asked for distance - test of bidirectional sending of data
			IF bPacket(PKT_BTNHI).bit5 and (LastButton(1).bit5 = 0) THEN		; (D) Distance
				; OK lets do a distance 
				gosub GetIRScanInCM[IRScanPin],b		; get it into some dummy variable.
				
				; then tell remote to display the value for us.
				; We will use simple version where we pass the number(byte) to remote as the equiv of packet number
				; but later will add capabilities to pass strings as extra bytes...
				gosub XbeeOutputVal[b]
			ENDIF 
	
			; For now on DIY will use keypad 0 and F but later may add special buttons...
			; For R1 and L1 we want to fire guns? Do two transistion.  One when button down
			; second when the button comes back up.
			IF bPacket(PKT_BTNHI).bit7 <> LastButton(1).bit7 THEN		; R1(F) test to see if button changed
				IF bPacket(PKT_BTNHI).bit7 THEN
					; Button Down - If wating for cycle to complete quickly end it, in case our internal switch fails to work...
					if RGunWaitingToCycle then
						hservo [RGUNP\-7000\0] 
					else
						hservo [RGUNP\7000\0] 
					endif
				else
					; Button up - for now two different quick sounds
					if not RGunWaitingToCycle then
						RGunWaitingToCycle = TRUE				; Button back up, will wait until gun fires to turn off
					else
						RGunWaitingToCycle = FALSE				; Don't need to wait any longer
					endif
				endif
			
			ENDIF 
			
			
			IF bPacket(PKT_BTNLOW).bit0 <> LastButton(0).bit0 THEN		; L1(0) test to see if button changed
				IF bPacket(PKT_BTNLOW).bit0 THEN
					; Button Down
					if LGunWaitingToCycle then
						hservo [LGUNP\-7000\0] 					; again failsafe if the gun internal switch fails
					else
						hservo [LGUNP\7000\0] 
					endif
				else
					if not LGunWaitingToCycle then
						LGunWaitingToCycle = TRUE				; Button back up, will wait until gun fires to turn off
					else
						LGunWaitingToCycle = FALSE				; don't need to wait for it to end any more
					endif
				endif
			ENDIF 
	
	
			IF (ABS(bPacket(PKT_LJOYLR)-128) > TravelDeadZone) THEN 
				; We also only process this if the turret servo is idle
				gethservo Turret,junk,idle
				if(idle)then
					TurretAngle = TurretAngle + ((bPacket(PKT_LJOYLR) - 128)*4) 
					IF TurretAngle > 12000 THEN 
						TurretAngle = 12000 
					ELSEIF TurretAngle < -12000 
						TurretAngle = -12000 
					ENDIF 
					;serout s_out,i14400,[sdec TurretAngle, 13] 
					IF fMainInput then		; If secondary call let the Movement function do the actual hservo update
						hservo [Turret\TurretAngle\0] 
					endif
				endif
			ENDIF 
	
			
		ENDIF
	  
	  	; bugbug - some buttons that are process may have to process here...
		IF fMainInput then		; must of the commands will only be processed by the main input
			LastButton(0) = bPacket(PKT_BTNLOW)
			LastButton(1) = bPacket(PKT_BTNHI)
		else
	;		define which bits we will process here (processed 0 and D, E,F)
	;					 76543210		
			LBMASK0	con	%00000001
			LBMASK1	con	%11100000
			LastButton(0) = (LastButton(0) & !LBMASK0) | (bPacket(PKT_BTNLOW) & LBMASK0)
			LastButton(1) = (LastButton(1) & !LBMASK1) | (bPacket(PKT_BTNHI) & LBMASK1)
		endif
	endif
return	


;==============================================================================
; Subroutine: ReadServoOffsets
; Will read in the zero points that wer last saved for the different servos
; that are part of this robot.  
;
;You must calibrate the servos to "zero". Each robot will be different! 
;When homed in and servos are at 0 degrees the robot should be standing 
;straight with the AtomPro chip pointing backward. If you know the number 
;of degrees the servo is off, you can calculate the value. 166.6 steps
;per degree. 
;
;==============================================================================
pT			var		pointer		; try using a pointer variable
cSOffsets	var		byte		; number of
bCSIn		var		byte
bCSCalc		var		byte		; calculated checksum

ReadServoOffsets:
	readdm 0, [cSOffsets, bCSIn]
	;serout s_out, i9600, ["RSO: cnt:", dec cSOffsets, " CS in:", hex bcsIn];

	if (cSOffsets > 0) and (cSOffsets <= NUMSERVOS) then 		; offset data is bad so go to error location

		; OK now lets read in the array of data
		readdm 2, [str aServoOffsets\csOffsets*2]
		
		;... calculate checksum...
		bCSCalc = 0
	
		for b = 0 to NUMSERVOS-1
			bCSCalc = AServoOffsets(b).lowbyte + AServoOffsets(b).highbyte
;			serout s_out, i9600, [" ", sdec aServoOffsets(i),":", hex aServoOffsets(i)]
		next
		
;		serout s_out, i9600, [ " CS Calc:", hex bCSCalc]
	
		if bCSCalc <> bCSIn then 
			aServoOffsets = rep 0\NUMSERVOS
		endif
	endif
;	serout s_out, i9600, [13]
	return


;==============================================================================
;[GetIRSensorInCM] - Get the distance to a target using a Sharp IR sensor.  These
; devices are not linear so we use a look up table to convert the results to CM.
; Parameters: IRPIN - Which pin to use
; Return: Distance in CM.  Range is from 0 to 80
;==============================================================================
; Define the table of values to convert
; This table is used to convert Sharp IR readings into CM
IRCMTable bytetable     80,80,80,80,80,80,80,80,80,78, | 
                        76,74,72,70,68,66,64,62,60,59, | 
                        58,57,55,53,52,51,50,49,48,47, | 
                        45,43,42,41,40,39,38,37,35,33, | 
                        32,31,30,30,29,29,28,28,27,27, | 
                        26,26,26,25,25,25,24,24,24,23, | 
                        23,22,22,21,21,20,20,20,19,19, | 
                        18,18,18,17,17,16,16,16,15,15, | 
                        15,14,14,13,13,13,12,12,11,11, | 
                        11,10,10,10,10,10,10,10,10,10 


IRPin		var byte
scanrange 	var word                 ' A/D result variable 


GetIRScanInCM[IRPIN]
                        
  ; first read in the IRvalue.
  adin IRPin,scanrange
   
  ; If it is greater than 512 it is out of range we will return 0
  IF scanrange > 512 THEN		; outside our table so return 0 to signal too close 
    return 0

  ENDIF 
  ; now convert entry to cm from table.  Old program
  ; divided value by 5.12 using floating point, but could simply multiply by 100 and divide by 512
  ; in integer math, which should be a lot quicker.  512*100 < 65536 so fits in word...
  return IRCMTable((scanrange*100)/512)


;==============================================================================
;[Movement] - This is the main Hservo interface.  This function takes new positions for each of the Legs servos
;             in 10s of a degree as well as the time we want the movement to take.  We then compute the speed
;             of each of the servos, such that all of the servos complete their moves at the same time.
;			  This function does not return until the servos have completed their movement.
;
; This version has been modified such that each of the movements will be done within a time of 50 time units.  That is when 
; the movement function is called with time > 50 the movement will be broken into Time/50 smaller moves.  During each segment, we
; will call out to a call back function that allows other inputs to be queried and change some of the servo locations or do what ever else
; that is necessary.
;Should never need to edit anything below this line.  Add user subroutines above this and below main. 
;==============================================================================
lefthippos1 		var slong 			; Passed in as .1 degrees then changed to HSERVO Units
leftkneepos1 		var slong 
leftanklepos1 		var slong 
righthippos1 		var slong 
rightkneepos1 		var slong 
rightanklepos1 		var slong 

last_lefthippos 	var slong 			; These are the last movement values kept in HSERVO units. - Set in SubMove
last_leftkneepos 	var slong 
last_leftanklepos 	var slong 
last_righthippos 	var slong 
last_rightkneepos 	var slong 
last_rightanklepos 	var slong 

MoveTime			var slong 			; this is the passed in movement time

SUBMOVETIME			con 100
cSubMoves			var	word			; count of sub moves for this move
dlefthippos			var slong 			; calculated deta movement for sub move for the left hip in HSERVO unit
dleftkneepos 		var slong 
dleftanklepos 		var slong 
drighthippos 		var slong 
drightkneepos 		var slong 
drightanklepos		var slong 

smlefthippos		var slong 			;Sub move left hip position in HSERVO units
smleftkneepos 		var slong 			
smleftanklepos 		var slong 
smrighthippos 		var slong 
smrightkneepos 		var slong 
smrightanklepos		var slong 


;movement [lefthippos1,leftkneepos1,leftanklepos1,righthippos1,rightkneepos1,rightanklepos1,speed1] 
movement [rightanklepos1,rightkneepos1,righthippos1,leftanklepos1,leftkneepos1,lefthippos1,MoveTime] 
#ifdef DEBUG_VERBOSE
	CBHervoCalled = FALSE		; did any call backs go an HSERVO?
#endif	
	; several step movements may be interrupted, so check flag and return.
	if fAbortCommand then
		return
	endif
	
	; First convert all of the servo positions into HSERVO values...
	lefthippos1 = (lefthippos1 * stepsperdegree1)/100
	leftkneepos1 = (leftkneepos1 * stepsperdegree1)/100
	leftanklepos1 = (leftanklepos1 * stepsperdegree1)/100
	righthippos1 = (righthippos1 * stepsperdegree1)/100
	rightkneepos1 = (rightkneepos1 * stepsperdegree1)/100
	rightanklepos1 = (rightanklepos1 * stepsperdegree1)/100
	
	
	; If the Move time is greater than our submove time we will loop through to do all but the last one
	; with calculated positions.  We will do the last one with the actual value.  This will also take
	; care of the case where the movetime is zero
	if (MoveTime > SUBMOVETIME) then

		; first lets get the delta sizes
		cSubMoves = (Movetime + SUBMOVETIME - 1)/SUBMOVETIME		; figure out how many sub moves there will be - Round up
		dlefthippos = (lefthippos1-last_lefthippos) / cSubMoves
		dleftkneepos = (leftkneepos1-last_leftkneepos) / cSubMoves
		dleftanklepos = (leftanklepos1-last_leftanklepos) /cSubMoves 
		drighthippos = (righthippos1-last_righthippos) / cSubMoves
		drightkneepos = (rightkneepos1-last_rightkneepos) / cSubMoves
		drightanklepos = (rightanklepos1-last_rightanklepos) / cSubMoves


		smlefthippos = last_lefthippos
		smleftkneepos = last_leftkneepos
		smleftanklepos = last_leftanklepos
		smrighthippos = last_righthippos
		smrightkneepos = last_rightkneepos
		smrightanklepos = last_rightanklepos
		
		while MoveTime > SUBMOVETIME
			smlefthippos = smlefthippos + dlefthippos
			smleftkneepos = smleftkneepos + dleftkneepos
			smleftanklepos = smleftanklepos + dleftanklepos
			smrighthippos = smrighthippos + drighthippos
			smrightkneepos = smrightkneepos + drightkneepos
			smrightanklepos = smrightanklepos + drightanklepos
		
			gosub Submovement[smrightanklepos, smrightkneepos, smrighthippos, smleftanklepos, smleftkneepos, smlefthippos, SUBMOVETIME]
			MoveTime = MoveTime - SUBMOVETIME
		wend
	endif

	; Last but not least we will call directly off with our acutal values to take care of any rounding.  If we were really time sensitive,
	; we could recompute times due to the overhead we introduced, but that probably is not necessary.
	gosub submovement [rightanklepos1,rightkneepos1,righthippos1,leftanklepos1,leftkneepos1,lefthippos1,MoveTime] 

	; and return
	return

;==============================================================================
;[SubMovement] - This is the subfunction to Movement that does all of the smaller movements
; and waiting and the like...
; local only to sub-move
;
;==============================================================================
; Parameters  to submove.
lefthippos 			var slong 			; these are the parameters passed into SubMove which are in HSERVO units
leftkneepos 		var slong 
leftanklepos 		var slong 
righthippos 		var slong 
rightkneepos 		var slong 
rightanklepos 		var slong 
SMoveTime			var	slong
longestmove 		var slong 			; Longest Move
MaxSpeed			var	slong			; this is the speed in 

lhspeed 			var slong 			; this is the computed speeds for the different servos
lkspeed 			var slong 
laspeed 			var slong 
rhspeed 			var slong 
rkspeed 			var slong 
raspeed 			var slong 


Submovement[rightanklepos,rightkneepos,righthippos,leftanklepos,leftkneepos,lefthippos,SMoveTime]

	; several step movements may be interrupted, so check flag and return.
	if fAbortCommand then
		return
	endif
	
	; If the Move time is zero then we are at our first call to movement, so simply set all of the speeds to zero and fall through to the 
	; main hservo... May extract this out later to be simply MovementInit...
	if SMoveTime = 0 then
		lhspeed=0; 
		lkspeed=0; 
		laspeed=0; 
		rhspeed=0; 
		rkspeed=0; 
		raspeed=0; 

	else	
		; Get the longest Move value
		gosub getlongest[lefthippos-last_lefthippos, | 
		           leftkneepos-last_leftkneepos, | 
		           leftanklepos-last_leftanklepos, | 
		           righthippos-last_righthippos, | 
		           rightkneepos-last_rightkneepos, | 
		           rightanklepos-last_rightanklepos],longestmove

		MaxSpeed = (longestmove/(SMoveTime/20)) 
		gosub getspeed[lefthippos,last_lefthippos,longestmove,MaxSpeed],lhspeed 
		gosub getspeed[leftkneepos,last_leftkneepos,longestmove,MaxSpeed],lkspeed 
		gosub getspeed[leftanklepos,last_leftanklepos,longestmove,MaxSpeed],laspeed 
		gosub getspeed[righthippos,last_righthippos,longestmove,MaxSpeed],rhspeed 
		gosub getspeed[rightkneepos,last_rightkneepos,longestmove,MaxSpeed],rkspeed 
		gosub getspeed[rightanklepos,last_rightanklepos,longestmove,MaxSpeed],raspeed
	endif 
   
	; BUGBUG: We will directly use the TurretAngle, that is not passed in.  We could change this a few ways.
	; one instead of waiting for the movement to complete after we do the HSERVO, we instead have our caller do
	; it before we do the hservo..., This would probably then move all of the call backs up to our parent, but then
	; that still relies on our parent coding it in...
   
   	; Now do our call off to HSERVO.  We have actual HSERVO values passed in so no computing needed.
	hservo [lefthip\-lefthippos + lefthip_start\lhspeed, | 
			righthip\righthippos + righthip_start\rhspeed, | 
			leftknee\-leftkneepos + leftknee_start\lkspeed, | 
			rightknee\rightkneepos + rightknee_start\rkspeed, | 
			leftankle\-leftanklepos + leftankle_start\laspeed, | 
			rightankle\rightanklepos + rightankle_start\raspeed, |
			Turret\TurretAngle\0] 
	

         
; Ok lets add in some debugging stuff to help later convert to integer math.
#ifdef DEBUG_VERBOSE
    serout s_out, i9600, [sdec SMoveTime, " ", sdec longestmove, " ", sdec MaxSpeed, "->",|
   		sdec -lefthippos + lefthip_start, ":", dec lhspeed, " ", | 
        sdec righthippos + righthip_start, ":", dec rhspeed, " ", | 
        sdec -leftkneepos + leftknee_start, ":", dec lkspeed, " ", | 
        sdec rightkneepos + rightknee_start, ":", dec rkspeed, " ", | 
        sdec -leftanklepos + leftankle_start, ":", dec laspeed, " ", | 
        sdec rightanklepos + rightankle_start, ":", dec raspeed, " ",|
        sdec TurretAngle, 13]
#endif         
         

;	This version adds a call back function to allow other things to be processed while this movement completes.
; we will loop through the list of servo numbers and if busy will call out else continue to the next servo.  This
; should keep it as fast as possible as a non-busy servo will not be checked again
	;   This version adds a call back function to allow other things to be processed while this movement completes. 
	do
	    gethservo lefthip,junk,idle
    	notidle= NOT idle
    	
	    gethservo righthip,junk,idle
	    notidle=notidle OR NOT idle

	    gethservo leftknee,junk,idle
	    notidle=notidle OR NOT idle

	    gethservo rightknee,junk,idle
	    notidle=notidle OR NOT idle

	    gethservo leftankle,junk,idle
	    notidle=notidle OR NOT idle
	    
	    gethservo rightankle,junk,idle
	    notidle=notidle OR NOT idle

		; If we are not idle do the call back
		if notidle then
			gosub MovementCB
		endif
	while (notidle and (not fAbortCommand))

Movement_DoReturn:
;   hservowait [lefthip,righthip,leftknee,rightknee,leftankle,rightankle]
	LHipAngle = -lefthippos
	RHipAngle = righthippos

   	; bugbug: if we abort we may need to do something different here???
	   
	last_lefthippos = lefthippos 
	last_leftkneepos = leftkneepos 
	last_leftanklepos = leftanklepos 
	last_righthippos = righthippos 
	last_rightkneepos = rightkneepos 
	last_rightanklepos = rightanklepos

; BUGBUG - Debug code to see timings.  Moved here to check to see what difference in timings are
; between when we call HSERVO during a call back and when we don't...
#ifdef DEBUG_VERBOSE
	gosub GetTimerVal[], wCurTimer
;	disable timeraint

	if CBHervoCalled then
		WMovCallsHServo = WMovCallsHServo + 1
		lMovDTSumHServo = lMovDTSumHServo + wCurTimer
	else
		wMovementCalls = wMovementCalls + 1	; number of times we called this.
		lMovementDTSum	= LMovementDTSum + wCurTimer
	endif
#endif
   return 

;==============================================================================
;[getlongest]
;==============================================================================
one var slong 
two var slong 
three var slong 
four var slong 
five var slong 
six var slong 
getlongest[one,two,three,four,five,six]
   if(one<0)then 
      one=-1*one 
   endif 
   if(two<0)then 
      two=-1*two 
   endif 
   if(three<0)then 
      three=-1*three 
   endif 
   if(four<0)then 
      four=-1*four 
   endif 
   if(five<0)then 
      five=-1*five 
   endif 
   if(six<0)then 
      six=-1*six 
   endif 
   if(one<two)then 
      one=two 
   endif 
   if(one<three)then 
      one=three 
   endif 
   if(one<four)then 
      one=four 
   endif 
   if(one<five)then 
      one=five 
   endif 
   if(one<six)then 
      one=six 
   endif 
   return one 
   
;==============================================================================
;[getspeed] Calculate the speed of a servo.  Note the servo positions
;           are passed in as 10s of a degree so we will then divide
;           the result by 10 to get us back to the proper speed unit
;==============================================================================
newpos var slong 
oldpos var slong 
longest var slong 
maxval var slong 
getspeed[newpos,oldpos,longest,maxval] 
   if(newpos>oldpos)then 
      return ((newpos-oldpos)/longest)*maxval
   endif 
   return ((oldpos-newpos)/longest)*maxval


