;==============================================================================
;Description: Simple program to configure an XBee radio to the desired settings
; 			to be used by the XBee DIY remote project.
;
; 			The program will first try to figure out what the current settings
;			are and the show them up on the terminal window.  It will then prompt
;			for a new value for MY(atmy) and for a title (ATNI)
;			After confirmation it will update the xbee to the desired baud rate
;			timeout, title, ...
;Programmer: Kurt (aka Kurte)
;
;KNOWN BUGS: 
;   - Probably Alot ;) 
;==============================================================================
USETASERIAL con 1		; use our timer serial support
; 

#ifndef BASICATOMPROARC32 ; Pro28 and Pro40 or pins 14 and 5
cXBEE_IN	con	p14		; The input pin
cXBEE_OUT	con	p15		; the output pin
;CXBEE_RTS	con	p7
#else; 		arc32
cXBEE_IN	con	p38		; The input pin
cXBEE_OUT	con	p36		; the output pin
;CXBEE_RTS	con	p7
#endif

cXBEE_OBAUD	con	0x4020	; i62500	; BUGBUG - not standard! - Is for HSERIAL

' variables used 
abBuf		var	byte(80)				; simple buffer
abMy		var	byte(10)				; Our my string
abNI		var	byte(80)				; Our Title string
abVR		var byte(10)				; Version number

fOK			var	byte					; only need bit, but...

wCurBaud	var	word
wCurTimeout	var	word					; current configuration
lTimeout	var	long
i			var	byte					; simple counter
cbRead		var	word					; count of how many bytes read
main:

;	high p4
;	high p5
;	high p6

	' lets try some of the standard settings and then try some of the settings we have used
	' in the past to find out the current settings.	
	pause 2000
	serout s_out, i9600, [13, 13, "XBee Configure program (Old Baud rate)", 13, |
			"  1 - 9600 (default new)", 13, |
			"  2 - 38400", 13,|
			"  3 - 57600 (* We use)", 13, |
			"  4 - 62500", 13, |
			"Choose: "]
	serin s_in, i9600, [dec i]
	
	if (i = 0) or (i > 4) then main
	
	if (i = 1) then
		serout s_out, i9600, ["Try i9600 Timeout: 2000", 13]
		gosub TryBaudTimeout[i9600, 2000], fOK		' standard unchanged xbee
	elseif (i = 2) 
		serout s_out, i9600, ["Try i38400 Timeout: 20", 13]
		pause 2000
		gosub TryBaudTimeout[i38400, 20], fOK	' standard but quck timeout xbee
	elseif (i = 3)
		serout s_out, i9600, ["Try i57600 Timeout: 20", 13]
		pause 1000
		gosub TryBaudTimeout[i57600, 20], fOK	' standard but quck timeout xbee
	elseif (i = 4)
		serout s_out, i9600, ["Try i62500 Timeout: 20", 13]
		pause 1000
		gosub TryBaudTimeout[cXBEE_OBAUD, 20], fOK	' standard but quck timeout xbee
	endif
	if not fOK then
		Serout s_out, i9600, [13, 13, "### Could not communicate with XBee ###", 13]
		pause 1000
		goto main
	endif

	serout s_out, i9600, ["We have current XBee config", 13]
	
	' Now lets ask for MY and NI...
AskChange:
	serout s_out, i9600, [13, "Current My: ", str abMY\10\13, ": ", str abNI\80\13, " Change (Y/N)? "]
	serin s_in, i9600, [str abBuf\80\13]
	if (abBuf(0) = "Y") or (abBuf(0) = "y") then
		' User wants to update the settings before writting
AskMy:		
		serout s_out, i9600, ["New MY(hex): "]
		serin s_in, i9600, [str abMY\10\13]
		' now verify string is valid hex
		for i = 0 to 9
			if i > 0 and abMY(i) = 13 then AskNI	' we have valid hex number
			if not ((abMY(i) >= "0" and abMY(i) <= "9") or (abMY(i) >= "a" and abMY(i) <= "f") or (abMY(i) >= "A" and abMY(i) <= "F")) then AskMy ' not valid char
		next
AskNI:	
		serout s_out, i9600, ["New NI Title: "]
		serin s_in, i9600, [str abNI\80\13]
		if abNI(0) = 13 then AskNI	' make sure not empty.

		' Ok go back up and ask if they want to change again
	endif

	if (abBuf(0) <> "N") and (abBuf(0) <> "n") then AskChange	' handles wrong input and after changes above...

	; now lets see what baud rate we want it set to now
AskNewBaud:	
	serout s_out, i9600, [13, 13, "New Baud Rate", 13, |
			"  1 - 9600 (Revert to 9600 new)", 13, |
			"  2 - 38400", 13,|
			"  3 - 57600 (* RTS)", 13, |
			"  4 - 62500 (* HSERIAL)", 13, |
			"Choose: "]
	serin s_in, i9600, [dec i]
	if (i < 1) or (i > 4) then AskNewBaud
	
	' We now have our changes so lets update the XBEE
	;low	CXBEE_RTS			; make sure that we can read.
	pause wCurTimeout
	
	serout cXBEE_OUT, wCurBaud, ["+++"]		; output the command header 
	
	lTimeout = wCurTimeout 
	lTimeout = ltimeout * 2000 *4			; convert to .5 us and 4 times the timeout...\

	serin cXBEE_IN, wCurBaud, ltimeout, TimedOutUpdate, [str abBuf\80\13]
	if (abBuf(0) <> "O") or (abBuf(1) <> "K") or (abBuf(2) <> 13) then  TimedOutUpdate
	
	; so now lets write out our new configuration information.
	
	; looks like we have it, now print out some of the configuration stuff
	serout cXBEE_OUT, wCurBaud, ["ATMY ", str abMY\10\13, 13, |		' output my
					"ATNI ", str abNI\80\13, 13, | 					' output NI
					"ATD6 0", 13]									' Turn off flow control - will turn on inside program
	if i = 1 then ; 9600
		serout cXBEE_OUT, wCurBaud, ["ATBD 3", 13, |				' Baud 9600
					"ATGT 3E8", 13]				 					' Long default guard time
	elseif i = 2	; 38400
		serout cXBEE_OUT, wCurBaud, ["ATBD 5", 13, |				' Baud 38400
					"ATGT 3", 13]				 					' Short guard time
	elseif i = 3 	; 57600
		serout cXBEE_OUT, wCurBaud, ["ATBD 6", 13, |				' Baud 57600
					"ATGT 3", 13]				 					' Short guard time
	else			; 62500
		serout cXBEE_OUT, wCurBaud, ["ATBD F424", 13, |				' Baud 62500 non-standard
					"ATGT 3", 13]				 					' Short guard time
	endif	
					
	serout cXBEE_OUT, wCurBaud, ["ATWR", 13, |						' Write out the changes
					"ATCN", 13]										' and exit command mode

	serout s_out, i9600, [13, 13, "*** XBee update Complete ***", 13]
	goto main
TimedOutUpdate:
	serout s_out, i9600, [13, 13, "### Error in trying to update XBee, will try to start over ###", 13]
	goto main
	stop	
	
	
;==============================================================================
; Function to find current baud rate and timeout for the XBEE
_wBaud		var	word
_wTimeout	var	word	
TryBaudTimeout[_wBaud, _wTimeout]:


	; ok lets try the pause
	; note in case flow control is enabled pull it down manaully to allow the thing to read.
	abni = rep 0\80
	;low	CXBEE_RTS			; make sure that we can read.
	pause _wtimeout
	
	serout cXBEE_OUT, _wbaud, ["+++"]		; output the command header 
	
	if _wTimeout = 20 then
		lTimeout = 40000	; 
	else
		ltimeout = 4000000
	endif	 
	gosub TASerin[cXBEE_IN, 0, _wBaud, @abBuf, 80, lTimeout, 13], cbRead
	if (cbRead = 0) then TimedOut
	if (abBuf(0) <> "O") or (abBuf(1) <> "K") or (abBuf(2) <> 13) then  TimedOut
	
	; looks like we have it, now print out some of the configuration stuff
	serout cXBEE_OUT, _wBaud, ["ATMY", 13]
	gosub TASerin[cXBEE_IN, 0, _wBaud, @abMY, 10, lTimeout, 13], cbRead
	if (cbRead = 0) or (abMy(cbRead-1) <> 13) then TimeoutMY

	serout cXBEE_OUT, _wBaud, ["ATNI", 13]
	gosub TASerin[cXBEE_IN, 0, _wBaud, @abNI, 80, lTimeout, 13], cbRead
	if (cbRead = 0) or (abNI(cbRead-1) <> 13) then TimedOutNI

	serout cXBEE_OUT, _wBaud, ["ATVR", 13]
	gosub TASerin[cXBEE_IN, 0, _wBaud, @abVR, 10, lTimeout, 13], cbRead
	if (cbRead = 0) or (abVR(cbRead-1) <> 13) then TimeoutVR

	serout cXBEE_OUT, _wBaud, ["ATCN", 13]

	serout s_out, i9600, ["atmy: ", str abMY\10\13, 13]
	serout s_out, i9600, ["atni: ", str abNI\80\13, 13]
	serout s_out, i9600, ["atvr: ", str abVR\10\13, 13]	
	; we have it.
	wCurBaud = _wBaud
	wCurTimeout = _wTimeout
	return 1
TimeoutVR:

TimedOutNI:
	serout s_out, i9600, ["atmy: ", str abMY\10\13, 13]
	for i= 0 to 79
		if (abNi(i) < " ") or (abni(i) > "|") then TO_NI2
	next
TO_NI2:			
	if i > 0 then
		serout s_out, i9600, ["partialatni: ", str abNI\i, 13]
	endif
TimeoutMY:
	if cbRead > 0 then
		serout s_out, i9600, ["partial my: ", str abMY\cbRead, 13]
	endif
TimedOut:
	return 0	