DEBUG con 1
;DEBUG_VERBOSE con 1
;DEBUG_ENTERLEAVE con 1

; lets try to configure XBEE to run on HSERIAL
XBEEONHSERIAL con 1

;Project Lynxmotion Rover including Robot Arm
;Description: Rover with 6DOF Robot Arm, controlled by a DIY remote with XBEE
;Software version: V2.0
;Date:
;Programmer: Jeroen Janssen (aka Xan)
;Modified by: Kurt (aka Kurte)
;
;Hardware setup: BB2 with ATOM 28 Pro, PS2 remote (See further down for connections)
;
;
;XBEE DIY CONTROLS:
;	- 0					Turn on/off rover/arm
;	- A					Select Rover
;	- B					Select Arm
;
;	ROVER CONTROL
;	Tank mode:
;	- Left Stick U/D	Left Wheels forward / reverse
;	- Right Stick U/D	Right Wheels forward / reverse
;
;	One Stick mode:
;	- Right Stick U/D	forward / reverse
;	- Right Stick L/R	steering
;
;	universal:
;	- 1-4				Choose Gear
;	- C					1 Stick Mode
;	- D					Tank Mode

;	ARM CONTROL
;;   
;   ARM IK Mode:
;	- Left Stick U/D	Gripper Up/Down
;	- Left Stick L/R	Gripper Away/Back
;	- Right Stick U/D	Gripper Angle Up/Down
;	- Right Stick L/R	Base Left/Right
;
;   ARM CAT Backhoe Mode:
;	- Left Stick U/D	Dipper (Elbow) angle
;	- Left Stick L/R	Rotation
;	- Right Stick U/D	Boom (Shoulder) Angle Up/Down
;	- Right Stick L/R	Bucket Curl (Wrist)
;
;	universal:
;	- C					ARM IK mode
;	- D					ARM Cat Backhoe mode
;	
;	- Left Slider		Wrist Rotate
;	- Right Slider		Gripper open/close
;	- Circle			Try turning on IR scanner servo and scan left and right at some rate and do distance testing...
;   - Square        	Debug - Turn debug prints on or off...
;
;
;KNOWN BUGS:
;	- None
;
;
;====================================================================
;[SETUP]
;Enable FSR Gripper 

EnableFSR con 2	;disable(0)/enable for 1023-0(1)/Enable for 0-1023(2) the FSR Gripper

;Select Arm type
;NOTE: Enable only one arm!
;AL5A con 1
;AL5B con 1
;AL5C con 1
AL5D con 1
;--------------------------------------------------------------------
;[TABLES]
GetACos bytetable 	255,253,252,251,250,248,247,246,244,243,242,241,239,238,237,235,234,233,232,230,229,228,226,|
					225,224,223,221,220,219,217,216,215,213,212,211,209,208,207,206,204,203,201,200,199,197,196,|
					195,193,192,191,189,188,186,185,184,182,181,179,178,177,175,174,172,171,169,168,166,165,163,|
					162,160,159,157,156,154,153,151,149,148,146,145,143,141,140,138,136,135,133,131,129,128,126,|
					124,122,120,118,116,115,113,111,109,106,104,102,100,98,96,93,91,89,86,83,81,78,75,73,70,66,63,|
					60,56,52,48,44,38,33,26,15,0

; This table is used to convert Sharp IR readings into CM
IRCMTable bytetable     80,80,80,80,80,80,80,80,80,78, | 
                        76,74,72,70,68,66,64,62,60,59, | 
                        58,57,55,53,52,51,50,49,48,47, | 
                        45,43,42,41,40,39,38,37,35,33, | 
                        32,31,30,30,29,29,28,28,27,27, | 
                        26,26,26,25,25,25,24,24,24,23, | 
                        23,22,22,21,21,20,20,20,19,19, | 
                        18,18,18,17,17,16,16,16,15,15, | 
                        15,14,14,13,13,13,12,12,11,11, | 
                        11,10,10,10,10,10,10,10,10,10 

MAXOBJECTDIST	con		40			; Maybe the longest that we can look in cm
MINOBJECTIR		con		180			; using table above index 35*5.12 = 179.2 round to something easy. Could calculate
MAXDELTADIST	con		3			; The largest difference in distance

; - Strings to pass back to remote control
_ROVERMODE	bytetable	10,"Rover Mode"
_ARMMODE	bytetable	8, "Arm Mode"
_1STICKMODE	bytetable	12,"1 Stick Mode"
_TANKMODE	bytetable	9, "Tank Mode"
_ARMIKMODE	bytetable	11,"Arm IK Mode"
_ARMCATMODE	bytetable	12,"Backhoe Mode"
_1stGear	bytetable	 10,"First Gear"
_2ndGear	bytetable	 8,"2nd Gear"
_3rdGear	bytetable	 8,"3rd Gear"
_4thGear	bytetable	 8,"4th Gear"
  
;--------------------------------------------------------------------
;[CONSTANDS]
c2DEC		con 100
c4DEC		con 10000

TRUE 		con 1
FALSE 		con 0

BUTTON_DOWN con 0
BUTTON_UP 	con 1

ROVER_MODE	con 0
ARM_MODE	con 1

TANK_MODE		con 0
ONE_STICK_MODE	con 1

ARMIK_MODE		con	0		; in the IK mode
ARMCAT_MODE		con 1		; In Cat Backhoe control mode

;calibrate steps per degree.
stepsperdegree1 con 1666 ; steps per degree in 10ths...
;--------------------------------------------------------------------
;[PIN NUMBERS]
BasePin 		con P0	;Base Connection
ShoulderPin 	con P1	;Shoulder Connection
ElbowPin  		con P2	;Elbow Connection
WristPin		con P3	;Wrist Connection
GripperPin		con P4	;Gripper Connection
WristRotatePin	con P5	;Wrist Rotation Connection (Optional)
IRScanPin		con P6	;IR Scanner pin (optional)


RightChannelPin	con P10	;Sabertooth Motor driver Right channel
LeftChannelPin  con P11	;Sabertooth Motor driver Left channel

GripperFSRPin	con P16	;Force Sensing Resistor for the Gripper
IRSensorPin		con	P18	;What pin is the IR Scanner on.
;--------------------------------------------------------------------
;[SERVO OFFSETS]
Base_Offset			con 416
Shoulder_Offset		con 3007
Elbow_Offset 		con -1192
Wrist_Offset 		con 2561
Gripper_Offset 		con 0
WristRotate_Offset 	con -1080
IRScan_Offset  		con 1250

MAXOFFSET			con	3500
; try to read in servo offsets from BAP EEPROM
NUMSERVOS	con 9
aServoOffsets	var	sword(NUMSERVOS)				
Base		con 0
Shoulder	con 1
Elbow		con 2
Wrist		con 3
Gripper		con 4
Rotate		con 5
IRScan		con 6
RMotor		con 7
LMotor		con 8

;--------------------------------------------------------------------
;[MOTOR NEUTRAL OFFSET]
RightChNeutral		con 1485;1465
LeftChNeutral		con 1485;1465
;--------------------------------------------------------------------
;[ANALOG SWITCHING POINT]
GripperHoldPressure	con 800 ;+/-1000 = neutral
;--------------------------------------------------------------------
;[MIN/MAX ANGLES] Degrees * 10
Base_MIN		con -900	;Mechanical limits of the Base
Base_MAX		con 900

Shoulder_MIN	con -780	;Mechanical limits of the Shoulder
Shoulder_MAX	con 900

Elbow_MIN		con -710	;Mechanical limits of the Elbow
Elbow_MAX		con 450

Wrist_MIN		con -900	;Mechanical limits of the Wrist
Wrist_MAX		con 900

Gripper_MIN		con -720	;Mechanical limits of the Gripper
Gripper_MAX		con 680

WristRotate_MIN	con -760	;Mechanical limits of the WristRotate
WristRotate_MAX	con 830
;--------------------------------------------------------------------
;[ARM DIMENSIONS AND INIT POSITIONS]
BaseLength		con 69	;Length of the Base [mm]

#IFDEF AL5A
  HumerusLength		con 94	;Length of the Humerus [mm]
  UlnaLength		con 108	;Length of the Ulna [mm]
  InitWristPosX 	con 30	;Init position X
  InitWristPosY 	con 120 ;Init position Y
  InitGripperAngle1 con -450;Init Angle gripper  
  
#ELSEIFDEF AL5B
  HumerusLength		con 119	;Length of the Humerus [mm]
  UlnaLength		con 127	;Length of the Ulna [mm]
  InitWristPosX 	con 20	;Init position X
  InitWristPosY 	con 120 ;Init position Y
  InitGripperAngle1 con -450;Init Angle gripper 
   
#ELSEIFDEF AL5C
  HumerusLength		con 156	;Length of the Humerus [mm]
  UlnaLength		con 154	;Length of the Ulna [mm]
  InitWristPosX 	con 20	;Init position X
  InitWristPosY 	con 120 ;Init position Y
  InitGripperAngle1 con -450;Init Angle gripper  
  
#ELSEIFDEF AL5D
  HumerusLength		con 147	;Length of the Humerus [mm]
  UlnaLength		con 187	;Length of the Ulna [mm] 
  InitWristPosX 	con  55; 55	;Init position X
  InitWristPosY 	con 120 ;Init position Y
  InitWristPosXP 	con  55; 55	;Init position X - Pre - may do init in two steps to keep arm from hanging up
  InitWristPosYP 	con 300 ;Init position Y
  InitGripperAngle1 con -100;Init Angle gripper
#ENDIF

; gripper length
  MetcarpalsLength	con	100; length [mm] from wrist servo to somewhere inside gripper... (using heavy duty rotate...)
  GripperLength		con	MetcarpalsLength

;--------------------------------------------------------------------
;[REMOTE]
DeadZone		con 5	;The deadzone for the analog input from the remote
;--------------------------------------------------------------------
;[IRScan]
SCAN_SERVO_MIN			con -600	; scan mimimum angle in .1 degrees
SCAN_SERVO_MAX			con 600		; scan maximum angle in .1 degrees
SCAN_SERVO_INC_LARGE	con	20		; Increment for each pass in .1 degrees
SCAN_SERVO_INC_SMALL	con	4		; Increment when we have found something
dxArmToSensorServo		con 80		; Delta X from Arm servo center to sensor servo center 
dyArmToSensorServo		con	10		; Delta Y from Arm base to center of sensor. BUGBUG:: currently Y to move arm after finding an object
IRSensorLength			con	45		; How far is the sensor from the center of the IRScan servo 

; These are difnitions for the object scan with the IR scanner.
LO_State 		var 	byte		; what state are we in for our scanning
LO_AngleDetect	var		slong		; What angle did we first find the object
LO_AngleLeft	var		slong		; Left edge
LO_ANGLERight	var		slong		; right edge
LO_Distance		var		word
LO_AngleInc		var		sword		; how much to increment by - May use large or small...

LOS_OFF			con 	0x0			; We need to do our init first
LOS_INIT		con		0x1
LOS_FINDOBJECT	con		0x2			; Object not found
LOS_FINDLEFTEDGE con	0x4			; We found object - scan plus directon to find edge
LOS_FINDRIGHTEDGE con	0x8			; We found one edge now scan different direction to find the other edge
LOS_GETCENTERDIST con	0x10		; OK lets go back to the center of the object and get the final distance.
LOS_DONE		con		0x80		; We are done!!!



;====================================================================
;[ANGLES]
BaseAngle1 			var sword	;Actual Angle of the Base servo
ShoulderAngle1 		var sword	;Actual Angle of the Schoulder servo
ElbowAngle1  		var sword	;Actual Angle of the Elbow servo
WristAngle1			var sword	;Actual Angle of the Wrist servo
Gripper1			var sword	;Actual Angle of the Gripper servo
WristRotateAngle1	var sword	;Actual Angle of the Wrist Rotation servo
;--------------------------------------------------------------------
;[POSITIONS]
WristPosX			var slong	;Position of the Gripper used for input
WristPosY			var slong
GlobalGripperAngle1	var sword	;Angle of the Gripper used for input
;--------------------------------------------------------------------
;[TIME]
MovementTime		var	word	; speed to do the next move at...

;--------------------------------------------------------------------
;[Gripper]
GripperPressure		var word	;Analog value from the FSR input
GripperPressureOld	var	word	;debug hold last value so we can print it out...
;--------------------------------------------------------------------
;[IR SCANNER]
IRDistance 			var word	;Distance from IR sensor in cm
IRDistanceCM		var	word	;Distance in CM
IRDistanceOld		var	word	; prev distance
IRSweepDir 			var bit		;Sweep direction 0=L/R, 1=R/L
IRSweepAngle		var	slong	; Our sweep angle in degrees...
IRSweepPos 			var slong	;Current position from the IR Servo
IRMoveDone			var	byte	; Last IR move done?
IRObjectX			var	slong	; Found object in Arm coordinate system
IRObjectY			var	slong	;
IRObjectZ			var	slong	;
IRObjectXGrip		var slong	; 
;--------------------------------------------------------------------
;[ROVER]
SteeringMode		var bit		;Switch between OneStick or Tank Steering
Gear				var byte	;Gear to set the motor speed 1 to 4
LStickX				var sword	;Stick position to calculate one stick steering
LStickY				var sword	;Stick position to calculate one stick steering
RStickX				var sword	;Stick position to calculate one stick steering
RStickY				var sword	;Stick position to calculate one stick steering
LDrive				var slong	;Left drive channel - converted to long for conversion to hservo
RDrive				var slong	;Right drive channel - converted to long for conversion to hservo
LDrivePrev			var	slong	; previous value...
RDrivePrev			var	slong

;--------------------------------------------------------------------
;[VARIABLES]

;GetArcCos
Cos4			var sword		;Input Cosinus of the given Angle, decimals = 4
AngleRad4		var sword		;Output Angle in radials, decimals = 4

;GetArcTan2
ArcTanX 		var sword		;Input X
ArcTanY 		var sword		;Input Y
ArcTan4			var slong		;Output ARCTAN2(X/Y)

;Arm Inverse Kinematics
IKWristPosX	    	var slong	;Input position of the Wrist X
IKWristPosY	    	var slong	;Input position of the Wrist Y
IKGripperAngle1		var slong	;Input Angle of the Gripper referring to the base
IKSW2				var long	;Length between Shoulder and Wrist
IKA14		    	var long	;Angle of the line S>W with respect to the ground in radians
IKA24		    	var long	;Angle of the line S>W with respect to the humerus in radians
Temp1				var long
Temp2				var long
IKSolutionWarning 	var bit		;Output true if the solution is NEARLY possible
IKSolutionError		var bit		;Output true if the solution is NOT possible
IKWristAngle1   	var sword	;Output Angle of Wrist in degrees
IKElbowAngle1   	var sword	;Output Angle of Elbow in degrees
IKShoulderAngle1   	var sword	;Output Angle of Shoulder in degrees
IKBaseAngle1		var sword	;Output Angle of Base in degrees

; BUGBUG: added for Xans ole version
IKGripperPosX		var	slong	; Location of the gripper X
IKGripperPosY		var	slong	; Location of the gripper Y
IKGripperPosZ		var	slong	; Location of the gripper Z
IKGripperPosYZ		var	slong	; Location of the gripper Z
;--------------------------------------------------------------------
; DIY remote data
bButtonsPrev		var	byte(2)	; don't update the buttons until later...
bPacketPrev			var	byte(8)	; Know if anything changed;
fPacketChanged		var	bit
i					var	byte	; temporary counter

;[Speed limitation] ; variables to limit max angle step to limit speed
TargetWristAngle1	var sword
MaxStepWristAngle	con 40
TargetGripper1		var sword
MaxStepGripper		con 40
;--------------------------------------------------------------------
;[GLOABAL]
ControlMode 	var bit			;Switch Between Rover and Arm
RoverOn			var bit			;Switch to turn on the Rover
Prev_RoverOn	var bit			;Previous loop state
ArmOn	 		var bit			;Switch to turn on the Arm
Prev_ArmOn		var bit			;Previous loop state 
ArmMode			var	bit			; IK mode or Cat backhoe mode
IRScanOn		var	bit			;Is the scanner on?
PrevIRScanOn	var	bit			;Previous scan on state
FDebugOn		var	bit			; BUGBUG:: Do we want to output debug information or not...
FPrevMoveDone	var	bit			;Has the previous Arm Move completed yet.
wCntFailedForced var	byte	; count of how many packets have we forced that still failed?
;====================================================================
;Interrupt init 

interruptenable var bit	;Memory to see if the interrupts are enabled
ENABLEHSERVO	;Enable Interrupts for HServo

low RightChannelPin	
low LeftChannelPin 
;====================================================================
;[INIT]
gosub InitTimer				; Initialize our timer (Timer A)
enable

gosub ReadServoOffsets		; get the offsets for the servos into our offset array

;Initialize servo positions
interruptenable=false
ArmMode = ARMIK_MODE		; default to IK mode
;Gosub InitArm

;Current gear
Gear = 3

;Initialize the controller
gosub InitController

ArmOn = FALSE
RoverOn = FALSE
FDebugOn = FALSE
IRScanOn = FALSE
PrevIRScanOn = FALSE
ControlMode = ROVER_MODE
LO_State = LOS_OFF				; Initialize to state of off.
IRSweepAngle = 0
HServo [IRScanPin\aServoOffsets(IRSCAN)]
RDrivePrev = 0xffffff	; value that should not be matched.
LDrivePrev = 0xffffff
;====================================================================
;[MAIN]	
main:

	;Read remote input
	GOSUB ControlInput
  
;[ROVER CALCULATIONS]
	IF ControlMode = ROVER_MODE THEN
		gosub RoverProcessInput 	; process the Controller inputs for Rover mode.
	
		;Drive Rover
		IF RoverOn THEN ;AND ArmOn=FALSE THEN
			IF Prev_RoverOn=0 THEN
		  		Sound P9,[60\4000,80\4500,100\5000]
			ENDIF

			; Warning updated Stick name values to that of UP/Down->Y and L/R ->X 		
			IF SteeringMode = TANK_MODE THEN	;Tanks steering mode
				LDrive = RStickY				; BUGBUG: the sticks feel better if you are behind and push up right stick...
				RDrive = LStickY     
			ELSE								;One button steering mode
			  	IF RStickY>=0 AND RStickX>=0 THEN ;Both positive - Forward maybe turning right
			  		RDrive = RStickY - RStickX
			    	LDrive = RStickY MIN RStickX
			    
			  	ELSEIF RStickY>=0 AND RStickX<0  ; Y positive/x- Forward turning left
			    	RDrive = RSticky MIN -RStickX
			    	LDrive = RStickY + RStickX      

			  	ELSEIF RStickY<0 AND RStickX>=0  ; Y neg x pos - Backup to right
			  		RDrive = -(-RStickY - RStickX)
			    	LDrive = -(-RStickY MIN RStickX)
			    	
			  	ELSEIF RStickY<0 AND RStickX<0  ;Both neg - backup to the left
			    	RDrive = -(-RSticky MIN -RStickX)
			    	LDrive = -(-RStickY + RStickX)      
		  
			  	ELSE
			    	RDrive = 0
			    	LDrive = 0      
			  	ENDIF
			
			ENDIF
			
			;Multiply speed with the selected gear
			RDrive = ((RDrive MAX 125) MIN -125) * 2*Gear 
			LDrive = ((LDrive MAX 125) MIN -125) * 2*Gear 
			
			;Send pulse to the Sabertooth
			;pulsout RightChannelPin, (RDrive+RightChNeutral*2)	; *0.5ms
			;pulsout LeftChannelPin,  (LDrive+LeftChNeutral*2)
			; RDrive and LDRIVE go from -1000 to 1000 (in 4th gear)
			if (RDrive <> RDrivePrev) or (LDrive <> LDrivePrev) then
				hservo [RightChannelPin\(RDrive+RightChNeutral)*16 - 24000, LeftChannelPin\(LDrive+LeftChNeutral)*16 - 2400]
				RDrivePrev = RDrive
				LDrivePrev = LDrive
#ifdef DEBUG
				serout s_out, i9600, ["RDrive: ", sdec RDrive,"(",sdec (RDrive+RightChNeutral)*16-24000, |
									") LDrive: ", sdec LDrive,"(",sdec (LDrive+LeftChNeutral)*16-24000,	")",13]
#endif
			endif									
			;pause 20
		ELSE
			;Send pulse to stop the Sabertooth
			IF Prev_RoverOn THEN
				Sound P9,[100\5000,80\4500,60\4000]
				;pulsout RightChannelPin, RightChNeutral*2	; *0.5ms
				;pulsout LeftChannelPin,  LeftChNeutral*2
				; converted to HSERVO.  On BAP28 conversion from pulse width centered at 1500 to hsevero range
				; can be done by (x*16)-24000
				hservo [RightChannelPin\(RightChNeutral*16)-24000, LeftChannelPin\(LeftChNeutral*16)-24000]
				;pause 20
			ENDIF
		ENDIF
		
		;Store previous ArmOn State
		IF RoverOn THEN
			Prev_RoverOn = 1
		ELSE
			Prev_RoverOn = 0
		ENDIF
	ENDIF
		

;[ARM CALCULATIONS]
	IF ControlMode = ARM_MODE THEN
		gosub ArmProcessInput 	; process the controller inputs for Rover mode.

		if ArmMode = ARMIK_MODE then	; we are in the Arm IK mode - default
			; only process the PS2 inputs if the arm is on...
			IF ArmOn THEN  
				; Process the left stick L-R
				if LStickX then
					IF IKSolutionWarning THEN ;Allow only movement to the base
						WristPosX = WristPosX - ((LStickX/18) min 0) ;X Position
					ELSE  
						WristPosX = WristPosX - LStickX/18 ;X Position
					ENDIF
				endif
				
				; Process the Left Stick U/D
				if LStickY then
					IF IKSolutionWarning THEN
						IF WristPosY > BaseLength THEN ;Allow only downward movement
					    	WristPosY = WristPosY - ((LStickY/18) min 0)	        
					  	ELSE ;Allow only upward movement
					    	WristPosY = WristPosY - ((LStickY/18) max 0)      
					  	ENDIF
					ELSE
					  WristPosY = WristPosY - LStickY/18 ;Y Position
					ENDIF
				ENDIF
							
				; set up the base angle
				BaseAngle1 = BaseAngle1 - RStickX/8	; 
				
				; last the Gripper angle
				GlobalGripperAngle1 = GlobalGripperAngle1 - RstickY/2  ; Rstick is already -
			endif
					
			;Inverse Kinematic calculations
			GOSUB ArmIK [WristPosX, WristPosY, GlobalGripperAngle1]  
			ShoulderAngle1 = -IKShoulderAngle1+900
			ElbowAngle1 = IKElbowAngle1+900
			WristAngle1 = -IKWristAngle1
			MovementTime = 50		; default speed for next move.
		else
			if ArmOn then
				; set up the base angle on Cat this is left joystick L/R
				BaseAngle1 = BaseAngle1 - LStickX/8	; 
				
				; Setup Dipper(Elbow) Angle Left joystick U/D
				ElbowAngle1 = ElbowAngle1 - LStickY/8 ; need to play with direction and speed...
				
				; Set Boom(Shoulder) angle with Right Joystick U/D
				ShoulderAngle1 = ShoulderAngle1 - RStickY/8	; again work on speed and direction
				
				; last the Bucket curl (Gripper angle) with right joystick L-R
				WristAngle1 = WristAngle1 - RstickX/2  ; Rstick ...
			endif
		endif
	
		GOSUB CheckLimits
		
		; We want to wait until the arm completes the last move before we try to calculate another arm location
		; however there may be other things we want to do with the time when the arm is busy.
		REPEAT
			GOSUB CheckForPrevMoveDone[FALSE], FPrevMoveDone ;Wait for servos to finish the previous move
			
			IF FPrevMoveDone THEN
			
				IF ArmOn THEN  
			    	IF ArmOn AND Prev_ArmOn=0 THEN
			      		Sound P9,[60\4000,80\4500,100\5000]
			      		gosub InitArm
			  
			      		GlobalGripperAngle1=0
			    	ENDIF
			
			    	GOSUB Movement[-BaseAngle1, -ShoulderAngle1, -ElbowAngle1, -WristAngle1, -Gripper1, -WristRotateAngle1, MovementTime]
			  	ELSE
			
			    	;Turn the bot off
			    	IF Prev_ArmOn THEN
			      		gosub InitArm    
			      		Sound P9,[100\5000,80\4500,60\4000]
			    	ENDIF
			  	ENDIF
			
				;Store previous ArmOn State
				IF ArmOn THEN
					Prev_ArmOn = 1
				ELSE
					Prev_ArmOn = 0
				ENDIF
			ENDIF    
				
			;[IR Scanner]
			; kje
			IF IRScanOn THEN
				gosub DoIRScan
			ENDIF
			
			if LO_STATE = LOS_DONE then
				GOSUB ArmIK3D [IRObjectX, IRObjectY, IRObjectZ]  
				
				; first we now want to update the variables that the PS2 manipulates else it will jerk it right back!
				; Also lets for now assume we want the gripper to level (0)
				GlobalGripperAngle1 = 0					; assume level.
			    TargetGripper1 = Gripper_MAX 			;Fully open Gripper
			
				wristPosX = IRObjectXGrip - MetcarpalsLength	; back the wrist off from object by the gripper length
				wristPosY = IRObjectY
				BaseAngle1 = -IKBaseAngle1;				; turns out this angle is opposite of scanner servo...
						
				; bugbug: Could build some of this directly into the ArmIK32 function.
				GOSUB ArmIK [wristPosX, wristPosY, GlobalGripperAngle1]  
				ShoulderAngle1 = -IKShoulderAngle1+900
				ElbowAngle1 = IKElbowAngle1+900
				WristAngle1 = -IKWristAngle1
				MovementTime = 1000		; do this at a slower speed...
				
				GOSUB CheckLimits ; probably want to check the limits here as well...
			
				FPrevMoveDone = FALSE		; force it to move the arm...
				LO_STATE = LOS_OFF
			
			ELSEIF LO_State <> LOS_OFF
			
				gosub LocateObjectWithIRScan
			ENDIF
			
		UNTIL FPrevMoveDone
	ENDIF
		
goto main
;====================================================================
;[INITARM] Initialization for the AL5x Arm
InitArm:
	; First see if we think we need to do a pre move to get to a 
	; safe place.  Also do at start as arm is in unknown state and
	; it could hang up on init
	if (WristposY < InitWristPosY) then
		; first try simple test...
		; BUGBUG: could cleanup code to not duplicate so much!
		;Gripper Positions
		WristPosX = InitWristPosXP		;Start positions of the Gripper
		WristPosY = InitWristPosYP
		GlobalGripperAngle1 = InitGripperAngle1
		  
		;Inverse Kinematic calculations
		GOSUB ArmIK [WristPosX, WristPosY, GlobalGripperAngle1]  
		ShoulderAngle1 = -IKShoulderAngle1+900
		ElbowAngle1 = IKElbowAngle1+900
		WristAngle1 = -IKWristAngle1
		
		GOSUB CheckLimits
		    
		IF interruptenable=FALSE THEN
			InterruptEnable=TRUE
			ENABLE
			;Movement time needs to be zero when first initializing HSERVO
			gosub movement [0,  -ShoulderAngle1,  -ElbowAngle1,  -WristAngle1,  0,  0,  0]   
		    
		ELSE
		  	gosub movement [0,  -ShoulderAngle1,  -ElbowAngle1,  -WristAngle1,  0,  0,  600]    
		ENDIF
		
		; wait for this move to complete before doing the main init
		GOSUB CheckForPrevMoveDone[TRUE], FPrevMoveDone ;Wait for servos to finish the previous move
	
	ENDIF
	
	;Gripper Positions
	WristPosX = InitWristPosX		;Start positions of the Gripper
	WristPosY = InitWristPosY
	GlobalGripperAngle1 = InitGripperAngle1
	
	;Inverse Kinematic calculations
	GOSUB ArmIK [WristPosX, WristPosY, GlobalGripperAngle1]  
	ShoulderAngle1 = -IKShoulderAngle1+900
	ElbowAngle1 = IKElbowAngle1+900
	WristAngle1 = -IKWristAngle1
	
	GOSUB CheckLimits
	
	IF interruptenable=FALSE THEN
		InterruptEnable=TRUE
		ENABLE
		;Movement time needs to be zero when first initializing HSERVO
		gosub movement [0,  -ShoulderAngle1,  -ElbowAngle1,  -WristAngle1,  0,  0,  0]   
	
	ELSE
		gosub movement [0,  -ShoulderAngle1,  -ElbowAngle1,  -WristAngle1,  0,  0,  600]    
	ENDIF
	
	pause 1000
	
return

;==============================================================================
; [InitController] - This function will attempt to initialize the XBee, it will check
; 			to see the power level of the VS to see if the power is turned on
;			if so it will init the XBee to the proper startup levels
;==============================================================================
InitController:
	bButtonsPrev = 0,0
	gosub InitXBee
	gosub SendXbeeNewDataOnlyPacket[1]				; make sure we are init to be new only mode
	return


;==============================================================================
; [ControlInput] - This function will call the XBEE support functions to
;			get the current information
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
ControlInput:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Control Input", 13]
#endif	

	; if we had valid input before save away the buttons state
	if fPacketValid then
		bButtonsPrev(0) = bPacket(0)
		bButtonsPrev(1) = bPacket(1)
	endif
	; First lets try getting a packet from the XBEE
	gosub ReceiveXBeePacket
	; See if we have a valid packet to process
	if fPacketValid  then
		; See if anything changed...
		fPacketChanged = FALSE	; don't need to worry about slop 
		for i = 0 to 7
			if (bPacket(i) <> bPacketPrev(i)) then
				fPacketChanged = TRUE
				bPacketPrev(i) = bPacket(i)
			endif
		next

		if fPacketChanged then
			if fPacketForced then
				gosub SendXbeeNewDataOnlyPacket[1]
			endif
#ifdef DEBUG
			serout s_out, i9600, ["P: ", hex bPacket(PKT_BTNLOW)\2, hex bPacket(PKT_BTNHI)\2, hex bPacket(PKT_RJOYLR)\2, hex bPacket(PKT_RJOYUD)\2, |
							     hex bPacket(PKT_LJOYLR)\2, hex bPacket(PKT_LJOYUD)\2, hex bPacket(PKT_RSLIDER)\2, hex bPacket(PKT_LSLIDER)\2, 13]
#endif
		endif

		; If the user presses the A key we will go into Rover Mode
		IF bPacket(PKT_BTNHI).bit2 and (bButtonsPrev(PKT_BTNHI).bit2 = 0) THEN		;A Button Rover Mode
	  		ControlMode = ROVER_MODE
	  		RoverOn=TRUE
			IF Prev_RoverOn=TRUE THEN ;Only beep when the rover is already on 
				sound P9,[50\5000,  50\4000]
	  		ENDIF
			gosub XBeeOutputString[@_ROVERMODE]
			
		ELSEIF bPacket(PKT_BTNHI).bit3 and (bButtonsPrev(PKT_BTNHI).bit3 = 0) 		;B Button Arm Mode
			ControlMode = ARM_MODE
			ArmOn = TRUE
			GlobalGripperAngle1=0 ;Put gripper horizontal
			IF Prev_ArmOn=TRUE THEN ;Only beep when the arm is already on
		  		sound P9,[50\4000,  50\5000]
		 	ENDIF
			gosub XBeeOutputString[@_ARMMODE]
		endif

		; warning changed meaning of L/RStickX/y	now L/r -> X right+ and U/D ->Y with up+
		; Take care of some deadband processing for the 4 inputs from the two joysticks on the PS2 remote.  
		; BUGBUG: could probably do in a for loop with updating 3-6 in place
		IF ABS(bPacket(PKT_LJOYLR) - 128) > DeadZone THEN ;Left Stick L/R
			LStickX = bPacket(PKT_LJOYLR) - 128 ;Wheels
		ELSE
			LStickX=0	    
		ENDIF
		
		IF ABS(bPacket(PKT_LJOYUD) - 128) > DeadZone THEN ;Left Stick U/D
			LStickY = bPacket(PKT_LJOYUD) - 128 ;Wheels
		ELSE
			LStickY=0
		ENDIF
		  
		IF ABS(bPacket(PKT_RJOYLR) - 128) > DeadZone THEN ;Right Stick L/R
			RStickX = bPacket(PKT_RJOYLR) - 128 ;Wheels
		ELSE
			RStickX=0
		ENDIF
		
		IF ABS(bPacket(PKT_RJOYUD) - 128) > DeadZone THEN ;Right Stick U/D
			RStickY = bPacket(PKT_RJOYUD) - 128 ;Wheels
		ELSE
			RStickY=0
		ENDIF
	endif
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Control Input ", hex fPacketValid, 13]
#endif	
	 
return

;--------------------------------------------------------------------
;[RoverProcessInput] Process the XBEE inputs for a rover.
RoverProcessInput:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Process Rover Input", 13]
#endif	
	; See if we have a valid packet
	if fPacketValid then
		wCntFailedForced = 0	; count of failed forced messages reset as we receive one.
		IF bPacket(PKT_BTNLOW).bit0 and (bButtonsPrev(PKT_BTNLOW).bit0 = 0) THEN	;Start Button (0 on keypad) test
	  		IF(RoverOn) THEN
	    		;Turn Rover off
	    		RoverOn = False
	  		ELSE
	  
	    		;Turn Rover on
	    		RoverOn = True	
	  		ENDIF
		ENDIF
		
		IF RoverOn THEN
			
			; We can directly choose our gear.
			IF bPacket(PKT_BTNLOW).bit1 and (bButtonsPrev(PKT_BTNLOW).bit1 = 0) THEN	;1 - first gear
				if Gear <> 1 then
					gosub XBeeOutputString[@_1stGear]
					Gear = 1 
			  		Sound P9,[50\3000]
				endif
			ElseIF bPacket(PKT_BTNLOW).bit2 and (bButtonsPrev(PKT_BTNLOW).bit2 = 0) 	;2 - 
				if Gear <> 2 then
					gosub XBeeOutputString[@_2ndGear]
					Gear = 2
			  		Sound P9,[50\3500]
				endif
			ElseIF bPacket(PKT_BTNLOW).bit3 and (bButtonsPrev(PKT_BTNLOW).bit3 = 0) 	;3 - 
				if Gear <> 3 then
					gosub XBeeOutputString[@_3rdGear]
					Gear = 3
			  		Sound P9,[50\4000]
				endif
			ElseIF bPacket(PKT_BTNLOW).bit4 and (bButtonsPrev(PKT_BTNLOW).bit4 = 0) 	;4 - 
				if Gear <> 4 then		
					gosub XBeeOutputString[@_4thGear]
					Gear = 4
			  		Sound P9,[50\4500]
				endif
			endif		
	
	
			; If the user presses the C key we will go into 1 Stick Mode
			IF bPacket(PKT_BTNHI).bit4 and (bButtonsPrev(PKT_BTNHI).bit4 = 0) THEN		;C Button 1 stick Mode
		  		SteeringMode = ONE_STICK_MODE
				gosub XBeeOutputString[@_1STICKMODE]
	
			ELSEIF bPacket(PKT_BTNHI).bit5 and (bButtonsPrev(PKT_BTNHI).bit5 = 0)		;D Button Tank Mode
		  		SteeringMode = TANK_MODE
				gosub XBeeOutputString[@_TANKMODE]
			ENDIF
		ENDIF
	else
		; Not a valid packet - we should go to a turned off state as to not roll into things!
		IF(RoverOn and fPacketForced) THEN
			wCntFailedForced = 	wCntFailedForced + 1	; increment the number of messages that failed
			if wCntFailedForced > 5 then
				RoverOn = False
			endif
	    endif
	endif
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Process Rover Input", 13]
#endif	

  	return

;--------------------------------------------------------------------
;[ArmProcessInput] Process the PS2 inputs for the arm

ArmProcessInput:
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Enter: Process Arm Input", 13]
#endif	

#IF EnableFSR > 0
	;Read analog input
	adin GripperFSRPin, GripperPressure

#if EnableFSR = 2
	GripperPressure = 1023 - GripperPressure		; convert 0->1023 into 1023->0
#endif	
	
;	if FDebugOn and (GripperPressure <> GripperPressureOld) then
;		serout s_out, i2400, ["Gripper: ", dec GripperPressure, 13]
;		GripperPressureOld = GripperPressure;
;	endif
#ELSE
	GripperPressure=1000
#ENDIF


	; See if we have a valid packet
	if fPacketValid then

		IF bPacket(PKT_BTNLOW).bit0 and (bButtonsPrev(PKT_BTNLOW).bit0 = 0) THEN	;Start Button (0 on keypad) test
			IF(ArmOn) THEN
				;Turn off
				ArmOn = False
			
				;Reset Gripper Positions
				WristPosX=55		;Reset IK postitions of the Gripper
				WristPosY=100
			
				BaseAngle1=0		;Reset servo angles.
				GlobalGripperAngle1=-550
				TargetGripper1=0	    	  
				Gripper1=0
				TargetWristAngle1=0
				WristRotateAngle1=0
			ELSE
		  
		    	;Turn on
		    	ArmOn = True	
		  
		    	;Put gripper horizontal
		    	GlobalGripperAngle1=0
		  	ENDIF
		ENDIF	
		
		IF ArmOn THEN
			IF bPacket(PKT_BTNHI).bit4 and (bButtonsPrev(PKT_BTNHI).bit4 = 0) THEN		;C Button Arm IK Mode
		  		ArmMode = ARMIK_MODE
				gosub XBeeOutputString[@_ARMIKMODE]
	
			ELSEIF bPacket(PKT_BTNHI).bit5 and (bButtonsPrev(PKT_BTNHI).bit5 = 0)		;D Button Arm Backhoe Mode
		  		ArmMode = ARMCAT_MODE
				gosub XBeeOutputString[@_ARMCATMODE]
			ENDIF	
	
			IF (bPacket(PKT_BTNLOW).bit2 = 0) and bButtonsPrev(PKT_BTNLOW).bit2 THEN	;2 Button test
				if (LO_STATE = LOS_OFF) or (LO_STATE = LOS_DONE) then
			  		LO_STATE = LOS_INIT		; BUGBUG: may want to cleanup more
				else
			  		LO_STATE = LOS_OFF		; turn it back off
				endif
			ENDIF
			
			
			IF (bPacket(PKT_BTNLOW).bit1 = 0) and bButtonsPrev(PKT_BTNLOW).bit1 THEN	;"1" Button test
				IRScanOn = !IRScanOn;
			
				; in either case lets set the servo back to the center location
				IRSweepAngle = 0
				HServo [IRScanPin\aServoOffsets(IRSCAN)]
				
				; if we turned it off, lets wait for it to get to this location and then turn off the power to that servo
				if IRScanON = 0 THEN
					HServoWait[IRScanPin]
					HServo [IRScanPin\-24000]		; should turn it off
				ENDIF
			ENDIF	
			
			;	bugbug: added debug output turn on or off.	
			IF (bPacket(PKT_BTNHI).bit7 = 0) and bButtonsPrev(PKT_BTNHI).bit7 THEN	; F Turn Debug on or off
				FDebugON = !FDebugON		; Invert the state of debug...
			ENDIF	
			
			; Wrist rotate 
			TargetWristAngle1 = WristRotate_MIN + ((WristRotate_MAX - WristRotate_MIN) * bPacket(PKT_RSLIDER))/256

			; Gripper
			TargetGripper1 = Gripper_min + ((Gripper_MAX - Gripper_MIN) * bPacket(PKT_LSLIDER))/256
	
			
			;Limit wristrotate max speed  
			IF (ABS(WristRotateAngle1-TargetWristAngle1) > MaxStepWristAngle) THEN
				IF ((WristRotateAngle1-TargetWristAngle1) >= 0) THEN
			  		WristRotateAngle1 = WristRotateAngle1 - MaxStepWristAngle
				ELSE
			  		WristRotateAngle1 = WristRotateAngle1 + MaxStepWristAngle	  	
				ENDIF
			ELSE
		  		WristRotateAngle1 = TargetWristAngle1
		 	ENDIF
		  
		  	;Limit gripper max speed
		  	IF (ABS(Gripper1-TargetGripper1) > MaxStepGripper) THEN
		    	IF ((Gripper1-TargetGripper1) >= 0) THEN
		      		IF (GripperPressure>GripperHoldPressure) THEN
		        		Gripper1 = Gripper1 - MaxStepGripper
		      		ENDIF
		  		ELSE
		      		Gripper1 = Gripper1 + MaxStepGripper	  	
		  		ENDIF
		  	ELSE
		  		Gripper1 = TargetGripper1
		  	ENDIF
		  
		ENDIF
	else
		; Not a valid packet - Lost control reset back to arm off mode...
		IF(ArmOn and fPacketForced) THEN
			;Turn off
			ArmOn = False
		
			;Reset Gripper Positions
			WristPosX=55		;Reset IK postitions of the Gripper
			WristPosY=100
		
			BaseAngle1=0		;Reset servo angles.
			GlobalGripperAngle1=-550
			TargetGripper1=0	    	  
			Gripper1=0
			TargetWristAngle1=0
			WristRotateAngle1=0
	    endif
	endif
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Process Arm Input", 13]
#endif	

return


;--------------------------------------------------------------------
;[GETARCCOS] Get the sinus and cosinus from the angle +/- multiple circles
;Cos4    	- Input Cosinus
;AngleRad4 	- Output Angle in AngleRad4
GetArcCos[Cos4]

	Cos4 = (Cos4 max c4DEC) min -c4DEC
	IF Cos4>=0 THEN
		AngleRad4 = GetACos(Cos4/78+1) ;78=table resolution (1/127)
		AngleRad4 = AngleRad4*62 ;62=acos resolution (pi/2/255)
	ELSE
		AngleRad4 = GetACos(-Cos4/78+1)  
		AngleRad4 = 31416-AngleRad4*62 ;62=acos resolution (pi/2/255)
	ENDIF

return AngleRad4
;--------------------------------------------------------------------
;[ARCTAN2] Gets the Inverse Tangus from X/Y with the where Y can be zero or negative
;ArcTanX 		- Input X
;ArcTanY 		- Input Y
;ArcTan4  		- Output ARCTAN2(X/Y)
GetArcTan2 [ArcTanX, ArcTanY]
	IF(ArcTanX = 0) THEN	; X=0 -> 0 or PI
		IF(ArcTanY >= 0) THEN
			ArcTan4 = 0
		ELSE
			ArcTan4 = 31415
		ENDIF
	ELSE	
		IF(ArcTanY = 0) THEN	; Y=0 -> +/- Pi/2
			IF(ArcTanX > 0) THEN
				ArcTan4 = 15707
			ELSE
				ArcTan4 = -15707
			ENDIF
		ELSE
			IF(ArcTanY > 0) THEN	;ARCTAN(X/Y)
				ArcTan4 = TOINT(FATAN(TOFLOAT(ArcTanX) / TOFLOAT(ArcTanY))*10000.0)
			ELSE	
				IF(ArcTanX > 0) THEN	;ARCTAN(X/Y) + PI	
					ArcTan4 = TOINT(FATAN(TOFLOAT(ArcTanX) / TOFLOAT(ArcTanY))*10000.0) + 31415
				ELSE					;ARCTAN(X/Y) - PI	
					ArcTan4 = TOINT(FATAN(TOFLOAT(ArcTanX) / TOFLOAT(ArcTanY))*10000.0) - 31415
				ENDIF
			ENDIF
		ENDIF
	ENDIF
return ArcTan4
;-------------------------------------------------------------------- 
;[ARM INVERSE KINEMATICS] Calculates the angles of the Humerus and Ulna for the given position of the Gripper
;IKGripperPosX		- Input position of the Gripper X
;IKGripperPosY		- Input position of the Gripper Y
;IKGripperPosZ		- Input Position of the Gripper Z
;IKGripperAngle1	- Input Angle of the Gripper referring to the base
;IKSolution			- Output true IF the solution is possible
;IKSolutionWarning 	- Output true IF the solution is NEARLY possible
;IKSolutionError	- Output true IF the solution is NOT possible
;IKUlnaAngle1  		- Output Angle of Ulna in degrees
;IKHumerusAngle1   	- Output Angle of Humerus in degrees
;IKBaseAngle1		- Output Angle of Base in degrees
;Some values changed to 1/4 or 1/2 to avoid overflowing
; BUGBUG: Debug variables...
IKWPXSave		var 	slong 
IKWPYSave	var		slong
IKGASave	var		slong
BaseAngle1Save	var	slong
ArmIK [IKWristPosX, IKWristPosY, IKGripperAngle1] 
    
	;IKSW - Length between Shoulder and wrist 
	IKSW2 = SQR((((IKWristPosY-BaseLength)*(IKWristPosY-BaseLength))+(IKWristPosX*IKWristPosX))*(c4DEC/4)) ; Value 1/2 what it should be   
	
	;IKA1 - Angle between SW line and the ground in rad 
	GOSUB GetArcTan2 [(IKWristPosY-BaseLength), IKWristPosX], IKA14 
	
	;IKA2 - Angle of the line S>W with respect to the humerus in radians 
	Temp1 = (((HumerusLength*HumerusLength) - (UlnaLength*UlnaLength))*(c4DEC/4) + (IKSW2*IKSW2)) ; Value is 1/4 of what it should be
	Temp2 = ((2*HumerusLength)*c2DEC * IKSW2) ; Value is 1/2 
	GOSUB GetArcCos [(Temp1 / (Temp2/c4DEC)) *2 ], IKA24 ; Corrected for 1/4 value / 1/2 value 
	
	;Elbow Angle 
	Temp1 = (((HumerusLength*HumerusLength) + (UlnaLength*UlnaLength))*(c4DEC/4) - (IKSW2*IKSW2)) ; Vale 1/4 what it should be
	Temp2 = (2*HumerusLength*Ulnalength) 
	GOSUB GetArcCos [(Temp1 / Temp2) * 4] ; Corredted for 1/4 
	IKElbowAngle1 = -(1800-AngleRad4*180/3141) 
	
	;Shoulder Angle 
	IKShoulderAngle1 = (IKA14 + IKA24) * 180 / 3141 
	
	;Wrist Angle 
	IKWristAngle1 = IKGripperAngle1-IKElbowAngle1-IKShoulderAngle1 
	   
	;Set the Solution quality
	IKSolutionWarning = ((IKSW2*2) > ((UlnaLength+HumerusLength-40)*c2DEC)) ; Corrected for 1/2 
	IKSolutionError = ((IKSW2*2) > ((UlnaLength+HumerusLength)*c2DEC)) ; Corrected for 1/2
	
	; BUGBUG: Debug output information to screen.
	if FDebugOn and ((IKWPXSave <> IKWristPosX) or (IKWPYSave <> IKWristPosY) or (IKGASave <> IKGripperAngle1) or (baseAngle1 <> baseangle1save)) then
		IKWPXSave = IKWristPosX
		IKWPYSave = IKWristPosY
		IKGASave = IKGripperAngle1
		BaseAngle1Save = BaseAngle1
		serout S_OUT, i2400, ["X=", sdec IKWPXSave, " Y=", sdec IKWPYSave, "Base Angle=", sdec baseangle1, " Grip=", sdec IKGASave, " WE:",hex IKSolutionWarning, hex IKSolutionError, 13]
	endif
return

#ifdef LATER
; Version of ArmIk that works to center of gripper instead of wrist...
IKWristY	var	slong
IKWristX	var	slong
ArmIK2 [IKGripperPosX, IKGripperPosY, IKGripperPosZ, IKGripperAngle1] 
    
   ;Length between the Base and Gripper 
   IKGripperPosYZ = SQR((IKGripperPosY*IKGripperPosY)+(IKGripperPosZ*IKGripperPosZ))
    
   ;Gripper offset 
   ;GOSUB GetSinCos [TOFLOAT(GripperAngle)]    
   ;IKWristX = TOINT(TOFLOAT(IKGripperPosX)-CosA*TOFLOAT(GripperLength)) 
   ;IKWristY = TOINT(TOFLOAT(IKGripperPosYZ)-(SinA*TOFLOAT(GripperLength)))-BaseLength 
   IKWristX = IKGripperPosX - (cos((IKGripperAngle1 * 256)/3600) * GripperLength)/128	; using integer math so convert to right locations...
   IKWristY = IKGripperPosYZ -(sin((IKGripperAngle1 * 256)/3600) * GripperLength)/128 - BaseLength 
   

   ;IKSW - Length between Shoulder and wrist 
   IKSW = sqr((IKWristY*IKWristY)+(IKWristX*IKWristX))
       
   ;IKA1 - Angle between SW line and the ground in rad 
   GOSUB GetBoogTan [IKWristY, IKWristX] 
   IKA1 = BoogTan 
    
   ;IKA2 - ? 
   IKA2 = FACOS((TOFLOAT((HumerusLength*HumerusLength) - (UlnaLength*UlnaLength)) + (IKSW*IKSW)) / (TOFLOAT(2*HumerusLength) * IKSW)) 
    
   ;Elbow Angle 
   ;IKSW = FACOS(0.61183);((TOFLOAT((HumerusLength*HumerusLength) + (UlnaLength*UlnaLength)) - (IKSW*IKSW)) / TOFLOAT(2*HumerusLength*Ulnalength)) 
    

   IKElbowAngle = (180-TOINT((FACOS((TOFLOAT((HumerusLength*HumerusLength) + (UlnaLength*UlnaLength)) - (IKSW*IKSW)) / TOFLOAT(2*HumerusLength*Ulnalength))*180.0) / 3.141592))*-1 
    

   ;Shoulder Angle 
   IKShoulderAngle = TOINT(((IKA1 + IKA2) * 180.0) / 3.141592) 
    
   ;Base Angle 
   GOSUB GetBoogTan [IKGripperPosZ, IKGripperPosY] 
   ;IKBaseAngle = TOINT((BoogTan*180.0) / 3.141592) 
    
   ;Wrist Angle 
   IKWristAngle = IKGripperAngle-IKElbowAngle-IKShoulderAngle 
       
   ;Set the Solution quality    
   IF(IKSW < TOFLOAT(UlnaLength+HumerusLength-30)) THEN 
      IKSolution = TRUE 
   ELSE 
      IF(IKSW < TOFLOAT(UlnaLength+HumerusLength)) THEN 
         IKSolutionWarning = TRUE 
      ELSE 
         IKSolutionError = TRUE    
      ENDIF 
   ENDIF 
    
return
#endif
;--------------------------------------------------------------------
;[CHECK ANGLES] Checks the mechanical limits of the servos
CheckLimits:
  BaseAngle1  = 	(BaseAngle1  min Base_MIN) max Base_MAX
  ShoulderAngle1 = 	(ShoulderAngle1 min Shoulder_MIN) max Shoulder_MAX
  ElbowAngle1 = 	(ElbowAngle1 min Elbow_MIN)  max Elbow_MAX
  WristAngle1  = 	(WristAngle1  min Wrist_MIN) max Wrist_MAX
  Gripper1 = 		(Gripper1 min Gripper_MIN) max Gripper_MAX
  WristRotateAngle1 =(WristRotateAngle1 min WristRotate_MIN)  max WristRotate_MAX
return
;--------------------------------------------------------------------
;[MOVEMENT] Does a servo groupmove to ensure that all servos reach
;the destination at the same time. 
BasePosition1			var slong
ShoulderPosition1 		var slong
ElbowPosition1 			var slong
WristPosition1			var slong
GripperPosition1		var slong
WristRotatePosition1	var slong
MoveTime 				var slong 
_mindex 				var byte
_mPrev_BasePos1			var slong
_mPrev_ShoulderPos1 	var slong
_mPrev_ElbowPos1  		var slong
_mPrev_WristPos1		var slong
_mPrev_GripperPos1		var slong
_mPrev_WristRotatePos1	var slong
_mBaseSpeed				var slong
_mShoulderSpeed 		var slong
_mElbowSpeed  			var slong
_mWristSpeed			var slong
_mGripperSpeed			var slong
_mWristRotateSpeed		var slong
_mMaxSpeed1				var slong
_mlongestmove1 			var slong 
Movement [BasePosition1,ShoulderPosition1,ElbowPosition1,WristPosition1,GripperPosition1,WristRotatePosition1,MoveTime] 


	if(MoveTime<>0)then 
		;Get the longest move
		gosub getlongest[BasePosition1	 -_mPrev_BasePos1, | 
						 ShoulderPosition1 -_mPrev_ShoulderPos1, | 
						 ElbowPosition1	 -_mPrev_ElbowPos1, | 
						 WristPosition1	 -_mPrev_WristPos1, | 
						 GripperPosition1	 -_mPrev_GripperPos1, | 
						 WristRotatePosition1 -_mPrev_WristRotatePos1], _mlongestmove1 
		
		; kje - converted to return MoveTime *10 - 
		_mMaxspeed1 = ((_mlongestmove1*stepsperdegree1)/(MoveTime/2)) 
		
		;Calculate speed for each servo
		gosub getspeed[BasePosition1,		_mPrev_BasePos1,	_mlongestmove1,		_mMaxSpeed1],	_mBaseSpeed
		gosub getspeed[ShoulderPosition1,	_mPrev_ShoulderPos1,_mlongestmove1,		_mMaxSpeed1],	_mShoulderSpeed 
		gosub getspeed[ElbowPosition1,	_mPrev_ElbowPos1,	_mlongestmove1,		_mMaxSpeed1],	_mElbowSpeed 
		gosub getspeed[WristPosition1,	_mPrev_WristPos1,	_mlongestmove1,		_mMaxSpeed1],	_mWristSpeed
		gosub getspeed[GripperPosition1,	_mPrev_GripperPos1,	_mlongestmove1,		_mMaxSpeed1],	_mGripperSpeed
		gosub getspeed[WristRotatePosition1,	_mPrev_WristRotatePos1,_mlongestmove1,_mMaxSpeed1],	_mWristRotateSpeed      

	else 
		_mBaseSpeed = 0
		_mShoulderSpeed = 0
		_mElbowSpeed = 0
		_mWristSpeed = 0
		_mGripperSpeed = 0
		_mWristRotateSpeed = 0
	endif 
	if FDebugOn and (_mlongestmove1 > 0) then
		serout s_out, i2400, ["Move: B=", sdec BasePosition1, "(", sdec _mBaseSpeed, |
							") S=", sdec ShoulderPosition1, "(", sdec _mShoulderSpeed, |
							") E=", sdec ElbowPosition1, "(", sdec _mElbowSpeed, |
							") W=", sdec WristPosition1,  "(", sdec _mWristSpeed,|
							") G=", sdec GripperPosition1, "(", sdec  _mGripperSpeed, |
							") R=", sdec WristRotatePosition1, "(", sdec _mWristRotateSpeed, |
							") Time: ", sdec  MoveTime, " MDelta:", sdec  _mlongestmove1," MSpeed: ", sdec _mMaxspeed1, 13]
	endif
   
	;Drive servos
	hservo [BasePin\		(BasePosition1*stepsperdegree1)/100 + aServoOffsets(BASE)\_mBaseSpeed, | 
	       ShoulderPin\	(ShoulderPosition1*stepsperdegree1)/100 + aServoOffsets(SHOULDER)\_mShoulderSpeed, | 
	       ElbowPin\	(ElbowPosition1*stepsperdegree1)/100 + aServoOffsets(Elbow)\_mElbowSpeed, | 
	       WristPin\	(WristPosition1*stepsperdegree1)/100 + aServoOffsets(Wrist)\_mWristSpeed, | 
	       GripperPin\	(GripperPosition1*stepsperdegree1)/100 + aServoOffsets(Gripper)\_mGripperSpeed, | 
	       WristRotatepin\(WristRotatePosition1*stepsperdegree1)/100 + aServoOffsets(Rotate)\_mWristRotateSpeed] 

return

;-------------------------------------------------------------------- 
;[ARM INVERSE KINEMATICS - 3D] Pass in X,Y, Z in rover/arm coordinates.  Handle rotate of base and then had off to
; above ArmIK...
;IKArmPosX		- Input position of the Gripper X
;IKArmPosY		- Input position of the Gripper Y
;IKArmPosZ		- Input Position of the Gripper Z
IKArmPosX	var		slong
IKArmPosY	var		slong
IKArmPosZ	var		slong
ArmIK3D [IKArmPosX, IKArmPosY, IKArmPosZ] 
	; The arm coordinates that were passed in are relative to the center of the arm pointing directly forward 
	; in the same plane as the wheels on the rover...  We will convert this into arm coordinates.  It will translate to
	; the base angle needed ans a Logical X value from the base.
	IRObjectXGrip = sqr((IKArmPosX * IKArmPosX) + (IKArmPosZ * IKArmPosZ))

  	;IKA1 - Get the angle of Base in Rads.
  	GOSUB GetArcTan2 [IKArmPosZ, IKArmPosX], IKA14 
	IKBaseAngle1 = (IKA14 * 180) / 3141 
	
	if FDebugOn then
    	serout S_OUT, i2400, ["IKArm3d X: ",sdec IKArmPosX, " Y: ", sdec IKArmPosY, " Z:", sdec IKArmPosZ, "->dist=", dec IRObjectXGrip, " Angle=",sdec IKBaseAngle1, 13]
	endif
return



;--------------------------------------------------------------------
;[CheckForPrevMoveDone] Loop that wait for the previous servo move to be finished
ForceWait	var	bit  
idle 		var byte
finished 	var byte
junk 		var word
CheckForPrevMoveDone[ForceWait]:
CheckAgain:
	finished = true
	gethservo BasePin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	gethservo ShoulderPin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	
	gethservo ElbowPin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	
	gethservo WristPin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	
	gethservo GripperPin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	
	gethservo WristRotatepin,junk,idle
	if(NOT idle)then
		finished=false
	endif	
	
	if(NOT finished)then 
		IF ForceWait THEN CheckAgain
		return FALSE
	ENDIF
	
	
	_mPrev_BasePos1			= BasePosition1
	_mPrev_ShoulderPos1 	= ShoulderPosition1
	_mPrev_ElbowPos1  		= ElbowPosition1
	_mPrev_WristPos1		= WristPosition1
	_mPrev_GripperPos1		= GripperPosition1
	_mPrev_WristRotatePos1	= WristRotatePosition1
	
	return TRUE

;--------------------------------------------------------------------
;[CHECKFORIRSCANMOVEDONE]
CheckForIRScanMoveDone:
	gethservo IRScanPin,junk,idle
	return idle
  
;--------------------------------------------------------------------
;[GETLONGEST] Returns the longest value
one 	var slong 
two 	var slong 
three 	var slong 
four 	var slong 
five 	var slong 
six 	var slong 
GetLongest[one,two,three,four,five,six] 
	if(one<0)then 
		one=-1*one 
	endif 
	if(two<0)then 
		two=-1*two 
	endif 
	if(three<0)then 
		three=-1*three 
	endif 
	if(four<0)then 
		four=-1*four 
	endif 
	if(five<0)then 
		five=-1*five 
	endif 
	if(six<0)then 
		six=-1*six 
	endif 
	if(one<two)then 
		one=two 
	endif 
	if(one<three)then 
		one=three 
	endif 
	if(one<four)then 
		one=four 
	endif 
	if(one<five)then 
		one=five 
	endif 
	if(one<six)then 
	  	one=six 
	endif 
return one  

;--------------------------------------------------------------------    
;[GetSpeed] Calculate the speed for each servo
newpos1 	var slong 
oldpos1 	var slong 
longest1 	var slong 
maxval1 	var slong 
GetSpeed[newpos1,oldpos1,longest1,maxval1] 

   if(newpos1>oldpos1)then 
      return (((newpos1-oldpos1)*maxval1)/longest1)/10
   endif 

return (((oldpos1-newpos1)*maxval1)/longest1)/10

;--------------------------------------------------------------------
;[GetIRSensorInCM]
IRPin		var byte
scanrange 	var word                 ' A/D result variable 


GetIRScanInCM[IRPIN]
                        
	; first read in the IRvalue.
	adin IRPin,scanrange
	
	; If it is greater than 512 it is out of range we will return 0
	IF scanrange > 512 THEN		; outside our table so return 0 to signal too close 
		return 0
	ENDIF 
	
	; now convert entry to cm from table.  Old program
	; divided value by 5.12 using floating point, but could simply multiply by 100 and divide by 512
	; in integer math, which should be a lot quicker.  512*100 < 65536 so fits in word...
	return IRCMTable((scanrange*100)/512)

;--------------------------------------------------------------------
;[ConvertIRToCM]
ConvertIRToCM[scanrange]:
   
	; If it is greater than 512 it is out of range we will return 0
	IF scanrange > 512 THEN		; outside our table so return 0 to signal too close 
		return 0
	ENDIF 
	
	; now convert entry to cm from table.  Old program
	; divided value by 5.12 using floating point, but could simply multiply by 100 and divide by 512
	; in integer math, which should be a lot quicker.  512*100 < 65536 so fits in word...
	return IRCMTable((scanrange*100)/512)

;--------------------------------------------------------------------
;[DOIRSCAN]
DoIRScan:
	; first see if the last Hservo for our pin has completed.  If not, bypass here and wait until next cycle
	gosub CheckForIRScanMoveDone, IRMoveDone
	IF IRMoveDone THEN
		; OK Move is done try reading from the scanner.
		gosub GetIRScanInCM[IRSensorPin], IRDistance
	
		if FDebugOn and (IRDistance <> IRDistanceOld) then
	  		serout s_out, i2400, ["IR in cm: ", dec IRDistance, " Angle:", sdec IRSweepAngle, 13]
	  		IRDistanceOld = IRDistance;
		endif
	
		; And setup to move the scanner again.
		; we will sweep from -60 to 60 degrees to start
		; lets try SCAN_SERVO_INC degrees to start off with
		
		if IRSweepDir then
			IF IRSweepAngle >= SCAN_SERVO_MAX THEN
		    	; At end change directions
		    	IRSweepDir = FALSE
		    	IRSweepAngle = SCAN_SERVO_MAX - SCAN_SERVO_INC_LARGE
		  	ELSE
		    	IRSweepAngle = IRSweepAngle + SCAN_SERVO_INC_LARGE
		  	ENDIF
		ELSE
			IF IRSweepAngle <= SCAN_SERVO_MIN THEN
		    	; At end change directions
		    	IRSweepDir = TRUE
		    	IRSweepAngle = SCAN_SERVO_MIN + SCAN_SERVO_INC_LARGE
		  	ELSE
		    	IRSweepAngle = IRSweepAngle - SCAN_SERVO_INC_LARGE
		  	ENDIF
		ENDIF
	
		; probably don't need to save in a variable, but maybe help debug  
		; also may want to play with speed...
		IRSweepPos = (IRSweepAngle*stepsperdegree1)/100 + aServoOffsets(IRSCAN)
		HServo [IRScanPin\IRSweepPos]
	ENDIF
	return

;--------------------------------------------------------------------
;[LOCATEOBJECTWITHIRSCAN]

LocateObjectWithIRScan:

	if LO_STATE = LOS_INIT then
		LO_STATE = LOS_FINDOBJECT		; set the state to we have not found anything yet.
    	HServo [IRScanPin\aServoOffsets(IRSCAN)]		; start always in the center
    	IRSweepAngle = 0			; start at angle 0
    	IRSweepDir = 0				; Start in one direction
    	LO_AngleInc = SCAN_SERVO_INC_LARGE ; start with large increments
    	IRDistanceOld = 0			; reset for the next run...
		return
	endif
	
	; first see if the last Hservo for our pin has completed.  If not, bypass here and wait until next cycle
	gosub CheckForIRScanMoveDone, IRMoveDone
	IF IRMoveDone  THEN
    	; OK Move is done try reading from the scanner.
    	;gosub GetIRScanInCM[IRSensorPin], IRDistance
    	; do simple AtoD, will convert to CM later...
		adin IRSensorPin,IRDistance
    	
		if FDebugOn and (IRDistance <> IRDistanceOld) and ((IRDistance > MINOBJECTIR) or (LO_STATE <> LOS_FINDOBJECT)) then
			serout s_out, i2400, ["IR in cm: ", dec IRDistance, " Angle:", sdec IRSweepAngle, 13]
			IRDistanceOld = IRDistance;
		endif
    	
    	; See what state we are in
    	if LO_State = LOS_FINDOBJECT then 		; ok we are still searching for an object
				
	  		; See if we found anything yet.
	  		if (IRDistance <> 0) and (IRDistance > MINOBJECTIR) then
	    		; Ok We have an object to work with, so lets remember where we are and try to find the edges
	    		LO_AngleDetect = IRSweepAngle
				LO_Distance = IRDistance;			; save away the distance
				LO_AngleInc = SCAN_SERVO_INC_SMALL	; lets do small increments.
				LO_STATE = LOS_FINDLEFTEDGE | LOS_FINDRIGHTEDGE	; we need to find both edges
			else
				; Have not found the object yet, lets continue scanning until we do.
				; Calculate the next angle to try.
				if IRSweepDir then
				  IF IRSweepAngle >= SCAN_SERVO_MAX THEN
				    ; At end change directions
				    IRSweepDir = FALSE
				    IRSweepAngle = SCAN_SERVO_MAX - LO_AngleInc
				  ELSE
				    IRSweepAngle = IRSweepAngle + LO_AngleInc
				  ENDIF
				ELSE
				  IF IRSweepAngle <= SCAN_SERVO_MIN THEN
				    ; At end change directions
				    IRSweepDir = TRUE
				    IRSweepAngle = SCAN_SERVO_MIN + LO_AngleInc
				  ELSE
				    IRSweepAngle = IRSweepAngle - LO_AngleInc
				  ENDIF
				ENDIF
				goto LocObj_DoHServo
			endif
			; we will end if here so we will fall through the next ifs to increment to next servo position...
		endif
		if LO_STATE = LOS_GETCENTERDIST then
			LO_STATE = LOS_DONE;		// OK we got our value so bail out of here
			
			; do a little validation of the last received distance.
;			if ABS((IRDistance - LO_Distance) > MAXDELTADIST) then
;				adin IRSensorPin,IRDistance
;		    	gosub GetIRScanInCM[IRSensorPin], IRDistance		; try getting it again to see if that helps
;				if ABS((IRDistance - LO_Distance) > MAXDELTADIST) then
					IRDistance = LO_Distance;		// Else fall back to where we found we were closeset...
					IRSweepAngle = LO_AngleDetect
;				endif
;			endif    	

			; now convert to CM
			gosub ConvertIRToCM[IRDistance], IRDistanceCM			
			
			; We now will calculate where the object is in arm coordinates...
			; Note Angle is in .1 degrees need to convert to funky angles used in Sin/Cos
			; first normalize the angle... (0-360)
			
			if IRSweepAngle < 0 then
				IRSweepAngle = 3600+IRSweepAngle;	// in 10ths of a degree
			endif

			; break up into smaller calcs to be able to debug better, reuse some variables here
			LO_AngleLeft = (IRDistanceCM *10) + IRSensorLength
			LO_AngleRight = (IRSweepAngle * 256 + 1800)/3600			; get the BAP angle in the range of 0-256 - do some round up stuff 

			IRObjectX = (cos(LO_AngleRight)*LO_AngleLeft + 64)/128  + dxArmToSensorServo ; the IR distance was in cm so make mm
			IRObjectZ = (sin(LO_AngleRight)*LO_AngleLeft + 64)/128 
			; for now assume some Y position
			IRObjectY = dyArmToSensorServo		; BUGBUG need to figure out something better
			if FDebugOn then
				serout s_out, i2400, ["LocObj: IR ", dec IRDistance, " in cm: ", dec IRDistanceCM, " Angle:", sdec IRSweepAngle, " DServo mm: ", sdec LO_AngleLeft, |
				                      " BAP angle:", sdec LO_AngleRight, " OX: ", sdec IRObjectX, " OY:", sdec IRObjectY, " OZ:", sdec IRObjectZ, 13]
			endif

			return
		else
			; We found something in range.  We want to continue in the current direction trying to find the minimum distance.  As a test we should see
			; that we don't fall of the object.  We need to try to find the minimum as the scanner will start to see the object before it is really on 
			; it...
			if IRDistance > LO_Distance then
	    		LO_AngleDetect = IRSweepAngle
				LO_Distance = IRDistance;			; save away the distance and the angle.
			endif
			
			if ((LO_DISTANCE - IRDistance) > MAXDELTADIST) or (IRSweepAngle >= SCAN_SERVO_MAX) or (IRSweepAngle <= SCAN_SERVO_MIN) then
				; Ok something changed, see which direction we are going
				if IRSweepDir then
					; Going right...
					LO_ANGLERight = IRSweepAngle		; save away where we stopped going right
					LO_STATE = LO_STATE & ~LOS_FINDRIGHTEDGE
				else
					LO_ANGLELeft = IRSweepAngle			; likewise where we stopped going left
					LO_STATE = LO_STATE & ~LOS_FINDLEFTEDGE
				endif
				if FDebugOn then
					serout s_out, i2400, ["LocObj findedge state:", hex LO_STATE, " Left:", sdec LO_AngleLeft, " Right: ",  sdec LO_AngleRight, " dir: ", |
										dec IRSweepDir, " Obj Angle: ", sdec LO_AngleDetect, " D: ", sdec LO_Distance, 13]
					IRDistanceOld = 0 ; try to get the other debug to trigger...
				endif
				
				; See if we have found both edges
				if LO_STATE = 0 then
					; We have our object lets bail.
					LO_STATE = LOS_GETCENTERDIST;
					IRSweepAngle = LO_AngleDetect 	; go back to where we found it closest
				else
					; no we still have an edge to go
					IRSweepDir = !IRSweepDir		; change direction
					IRSweepAngle = (IRSweepAngle + LO_ANGLEDetect)/2	; Start the scan half way from where we found min and it fell off.
				endif
			
			else
				; still in object continue to scan...						
			    if IRSweepDir then
			        IRSweepAngle = IRSweepAngle + LO_AngleInc
			    ELSE
			        IRSweepAngle = IRSweepAngle - LO_AngleInc
			    ENDIF
			endif
		endif

		; probably don't need to save in a variable, but maybe help debug  
    	; also may want to play with speed...
LocObj_DoHServo:    	
    	IRSweepPos = (IRSweepAngle*stepsperdegree1)/100 + aServoOffsets(IRSCAN)
    	HServo [IRScanPin\IRSweepPos]
	ENDIF
	return


;==============================================================================
;[ReadServoOffsets]
; Will read in the zero points that wer last saved for the different servos
; that are part of this robot.  
;
;==============================================================================
cSOffsets	var		byte		; number of
bCSIn		var		byte
bCSCalc		var		byte		; calculated checksum

ReadServoOffsets:
	readdm 0, [cSOffsets, bCSIn]
;	serout s_out, i9600, ["RSO: cnt:", dec cSOffsets, " CS in:", hex bcsIn];
	if (cSOffsets = 0) or (cSOffsets > NUMSERVOS) then RSO_Error		; offset data is bad so go to error location

	; OK now lets read in the array of data
	readdm 2, [str aServoOffsets\csOffsets*2]
	
	;... calculate checksum...
	; for fun lets do in assembly language.
	mov.l	#ASERVOOFFSETS, er1		; get the address of our array into er1
	mov.b	@CSOFFSETS, r0h			; get the read in count of entries into our counter
	shal.b	r0h						; multiply by 2 for the number of bytes (2 bytes per entry)
	xor.b	r0l, r0l				; zero out checksum
RSO_LOOP:
	mov.b	@er1+, r2l				; get the next byte and post increment the pointer
	add.b	r2l, r0l				; add it to our checksum
	dec.b	r0h						; decrement our count
	bne		RSO_LOOP:8				; did not go to zero go to process next byte
	mov.b	r0l,@BCSCALC:24			; save away our calculated checksum

; verify the values look ok...
	for temp1 = 0 to numservos-1
		if (aServoOffsets(temp1) > MAXOFFSET) or (aServoOffsets(temp1) < -MAXOFFSET) then RSO_ERROR;
;		serout s_out, i9600, [" ", sdec aServoOffsets(i),":", hex aServoOffsets(i)]
	next
;	serout s_out, i9600, [ " CS Calc:", hex bCSCalc]
	;and verify that the checksum looks ok
	if bCSCalc = bCSIn then RSO_RETURN
	
RSO_ERROR:
	aServoOffsets = BASE_OFFSET,SHOULDER_OFFSET,ELBOW_OFFSET,WRIST_OFFSET,GRIPPER_OFFSET,WRISTROTATE_OFFSET,IRSCAN_OFFSET, RightChNeutral, LeftChNeutral	
RSO_RETURN:	
;	serout s_out, i9600, [13]
	return
	
